
void Init_Aradas(void)
{
// this function loads and initializes the aradas to a known state
	
int frame;  // looping var

// create the first bob
Create_BOB(&aradas[0],0,0,64,64,32,
            BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME,
            DDSCAPS_SYSTEMMEMORY);

	// load animation frames 
	Load_Bitmap_File(&bitmap8bit, "ART/arada.bmp");

	for (frame=0; frame < 31; frame++)
		Load_Frame_BOB(&aradas[0],&bitmap8bit,frame,frame%8,frame/8,BITMAP_EXTRACT_MODE_CELL);  

    // unload data infile
    Unload_Bitmap_File(&bitmap8bit);


// set state to off
aradas[0].state                       = ARADA_STATE_DEAD;
aradas[0].varsI[INDEX_ARADA_DIR]    = 0;
aradas[0].varsI[INDEX_ARADA_DAMAGE] = 0;

// make copies
for (int ship=1; ship < MAX_ARADAS; ship++)
    {
    memcpy(&aradas[ship], &aradas[0], sizeof(BOB));
    } // end for arada

} // end Init_Aradas

//////////////////////////////////////////////////////////////////////////////////////

void Reset_Aradas(void)
{
// this function resets all the aradas

for (int ship=0; ship < MAX_ARADAS; ship++)
    {
    aradas[ship].state                       = ARADA_STATE_DEAD;
    aradas[ship].varsI[INDEX_ARADA_DIR]    = 0;
    aradas[ship].varsI[INDEX_ARADA_DAMAGE] = 0;
    } // end for arada

} // end Reset_Aradas

///////////////////////////////////////////////////////////

void Start_Arada(int override=0, int x=0, int y=0, int dir=0)
{
// this functions starts an arada, note that if override = 1
// then the function uses the sent data otherwise it's random

// first find an available arada
for (int index=0; index < MAX_ARADAS; index++)
	{
	// is this one dead
	if (aradas[index].state  == ARADA_STATE_DEAD)
		{
		// position the arada on a circle ring randomly from player

		int ang = rand()%16;
		float xpos = ARADA_RANGE_RING*cos_look16[ang];
		float ypos = ARADA_RANGE_RING*sin_look16[ang];

        // set position
		aradas[index].varsI[INDEX_WORLD_X] = player_x+xpos;
		aradas[index].varsI[INDEX_WORLD_Y] = player_y+ypos;

		// set direction to right
		aradas[index].varsI[INDEX_ARADA_DIR] = ang;

		// set velocity
		aradas[index].xv = 4+rand()%8;
		aradas[index].yv = RAND_RANGE(-4,4);

		
		// set remaining state variables
		aradas[index].state  = ARADA_STATE_ALIVE;
        aradas[index].varsI[INDEX_ARADA_DIR] = 0;
        aradas[index].varsI[INDEX_ARADA_DAMAGE] = 0;

		// done so exit
		return;

		} // end if
	
	} // end for index

} // end Start_Arada

////////////////////////////////////////////////////////////

void Move_Aradas(void)
{
// this function moves all the aradas and does the ai

for (int index=0; index < MAX_ARADAS; index++)
    {
    // test if plasma pulse is in flight
    if (aradas[index].state == ARADA_STATE_ALIVE)
        {
        // move the arada
        aradas[index].varsI[INDEX_WORLD_X]+=aradas[index].xv;
        aradas[index].varsI[INDEX_WORLD_Y]+=aradas[index].yv;
          
        // test for boundaries
        if (aradas[index].varsI[INDEX_WORLD_X] > UNIVERSE_MAX_X)
           {
  		   aradas[index].varsI[INDEX_WORLD_X] = UNIVERSE_MIN_X;
           } // end if
		else
		if (aradas[index].varsI[INDEX_WORLD_X] < UNIVERSE_MIN_X )
           {
  		   aradas[index].varsI[INDEX_WORLD_X] = UNIVERSE_MAX_X;
           } // end if	    

		if (aradas[index].varsI[INDEX_WORLD_Y] > UNIVERSE_MAX_Y)
           {
		   aradas[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MIN_Y;
           } // end if
		else
		if (aradas[index].varsI[INDEX_WORLD_Y] < UNIVERSE_MIN_Y ) 
           {
		   aradas[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MAX_Y;
           } // end if


       // leave particle trail
    if (aradas[index].varsI[INDEX_ARADA_DAMAGE] > (MAX_ARADA_DAMAGE >> 2) && 
       (rand()%3)==1)
        {
        Start_Particle(PARTICLE_TYPE_FLICKER, PARTICLE_COLOR_WHITE, 30+rand()%25, 
                      aradas[index].varsI[INDEX_WORLD_X]+RAND_RANGE(-4,4), 
                      aradas[index].varsI[INDEX_WORLD_Y]+RAND_RANGE(-4,4), 
                      (aradas[index].xv*.125), (aradas[index].yv*.125));
    
        Start_Particle(PARTICLE_TYPE_FADE, PARTICLE_COLOR_RED, 5, 
                      aradas[index].varsI[INDEX_WORLD_X]+RAND_RANGE(-4,4), 
                      aradas[index].varsI[INDEX_WORLD_Y]+RAND_RANGE(-4,4), 
                      (aradas[index].xv*.125), (aradas[index].yv*.125));


    
        } // end if

		// do ai
		
		// tracking algorithm

        // compute vector toward player
        float vx = player_x - aradas[index].varsI[INDEX_WORLD_X];
        float vy = player_y - aradas[index].varsI[INDEX_WORLD_Y];

        // normalize vector (sorta :)
        float length = Fast_Distance_2D(vx,vy);


        // only track if reasonable close
        if (length < MIN_ARADA_TRACKING_DIST)
            {
            vx=arada_tracking_rate*vx/length;
            vy=arada_tracking_rate*vy/length;

            // add velocity vector to current velocity
            aradas[index].xv+=vx;
            aradas[index].yv+=vy;

            // add a little noise
            if ((rand()%10)==1)
                {
                vx = RAND_RANGE(-1,1);
                vy = RAND_RANGE(-1,1);
                aradas[index].xv+=vx;
                aradas[index].yv+=vy;
                } // end if

            // test velocity vector of aradas
            length = Fast_Distance_2D(aradas[index].xv, aradas[index].yv);

            // test for velocity overflow and slow
            if (length > MAX_ARADA_VELOCITY)
                {
                // slow down
                aradas[index].xv*=0.75;
                aradas[index].yv*=0.75;

                } // end if


        else
            {
            // add a random velocity component
            if ((rand()%30)==1)
                {
                vx = RAND_RANGE(-2,2);
                vy = RAND_RANGE(-2,2);

                // add velocity vector to current velocity
                aradas[index].xv+=vx;
                aradas[index].yv+=vy;

                // test velocity vector of aradas
                length = Fast_Distance_2D(aradas[index].xv, aradas[index].yv);

                // test for velocity overflow and slow
                if (length > MAX_ARADA_VELOCITY)
                    {
                    // slow down
                    aradas[index].xv*=0.75;
                    aradas[index].yv*=0.75;

                    } // end if
        
                } // end if
           
            } // end else

        } // end if activated
    else
        {
        // compute dist from player
        float vx = player_x - aradas[index].varsI[INDEX_WORLD_X];
        float vy = player_y - aradas[index].varsI[INDEX_WORLD_Y];

        float length = Fast_Distance_2D(vx,vy);
		}

            // turn turret always
            // compute d0, d1, d2   

            // first create a vector point in the direction of the turret
            
            // compute current turrent vector
            int tdir1 = aradas[index].varsI[INDEX_ARADA_DIR];
            
            float d1x = aradas[index].varsI[INDEX_WORLD_X] + cos_look16[tdir1]*32;
            float d1y = aradas[index].varsI[INDEX_WORLD_Y] + sin_look16[tdir1]*32;

            // compute turret vector plus one
            int tdir2 = aradas[index].varsI[INDEX_ARADA_DIR]+1;
            
            if (tdir2 > 15) 
                tdir2 = 0;

            float d2x = aradas[index].varsI[INDEX_WORLD_X] + cos_look16[tdir2]*32;
            float d2y = aradas[index].varsI[INDEX_WORLD_Y] + sin_look16[tdir2]*32;
            
            // compute turret vector minus one
            int tdir0 = aradas[index].varsI[INDEX_ARADA_DIR]-1;
            
            if (tdir0 < 0) 
                tdir0=15;

            float d0x = aradas[index].varsI[INDEX_WORLD_X] + cos_look16[tdir0]*32;
            float d0y = aradas[index].varsI[INDEX_WORLD_Y] + sin_look16[tdir0]*32;

            // now find the min dist
            float dist0 = Fast_Distance_2D(player_x - d0x, player_y - d0y);
            float dist1 = Fast_Distance_2D(player_x - d1x, player_y - d1y);
            float dist2 = Fast_Distance_2D(player_x - d2x, player_y - d2y);

            if (dist0 < dist2 && dist0 < dist1)
                {
                // the negative direction is best        
                aradas[index].varsI[INDEX_ARADA_DIR] = tdir0;

                } // end if
            else
            if (dist2 < dist0 && dist2 < dist1)
                {
                // the positive direction is best
                aradas[index].varsI[INDEX_ARADA_DIR] = tdir2;

                } // end if

            // test for firing
            if ( (rand()%25)==1 &&
				(Fast_Distance_2D(player_x-aradas[index].varsI[INDEX_WORLD_X],
			                 player_y-aradas[index].varsI[INDEX_WORLD_Y]) < ARADA_MIN_ATTACK_RANGE) )
                {   
					int plasma_x = aradas[index].varsI[INDEX_WORLD_X];
                    int plasma_y = aradas[index].varsI[INDEX_WORLD_Y];


                    float plasma_xv = aradas[index].xv+cos_look16[aradas[index].varsI[INDEX_ARADA_DIR]]*PLASMA_SPEED_ARADA;
	                float plasma_yv = aradas[index].yv+sin_look16[aradas[index].varsI[INDEX_ARADA_DIR]]*PLASMA_SPEED_ARADA;

                    // fire weapon
                    Fire_Plasma(plasma_x, plasma_y, plasma_xv,plasma_yv,PLASMA_ANIM_ENEMY);

				} // end if fire

			//} // end if

		} // end if alive

	} // end for index

} // end Move_Aradas

///////////////////////////////////////////////////////////

void Draw_Aradas(void)
{
// this function moves all the aradas

for (int index=0; index < MAX_ARADAS; index++)
    {
    // test if plasma pulse is in flight
    if (aradas[index].state == ARADA_STATE_ALIVE)
        {
        // transform to screen coords
        aradas[index].x = aradas[index].varsI[INDEX_WORLD_X] - (aradas[index].width >> 1) - player_x + (SCREEN_WIDTH/2);
        aradas[index].y = aradas[index].varsI[INDEX_WORLD_Y] - (aradas[index].height >> 1) - player_y + (SCREEN_HEIGHT/2);

        // select the proper frame
		aradas[index].curr_frame = (aradas[index].varsI[INDEX_ARADA_DIR] + 16*(rand()%2) );
        // draw the arada
        Draw_BOB(&aradas[index],lpddsback);
         
        } // end if

    } // end for index

} // end Draw_Aradas

////////////////////////////////////////////////////////////

void Delete_Aradas(void)
{
// this function simply deletes all memory and surfaces
// related to the aradas

for (int index=0; index < MAX_ARADAS; index++)
    Destroy_BOB(&aradas[index]);

} // end Delete_Aradas

///////////////////////////////////////////////////////////
