
///////////////////////////////////////////////////////////

void Init_Kraits(void)
{
// this function loads and initializes the kraits to a known state
	
int frame;  // looping var

// create the first bob
Create_BOB(&kraits[0],0,0,64,64,32,
            BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME,
            DDSCAPS_SYSTEMMEMORY);

	// load animation frames 
	Load_Bitmap_File(&bitmap8bit, "ART/krait.bmp");

	for (frame=0; frame < 31; frame++)
		Load_Frame_BOB(&kraits[0],&bitmap8bit,frame,frame%8,frame/8,BITMAP_EXTRACT_MODE_CELL);  

    // unload data infile
    Unload_Bitmap_File(&bitmap8bit);


// set state to off
kraits[0].state                       = KRAIT_STATE_DEAD;
kraits[0].varsI[INDEX_KRAIT_DIR]    = 0;
kraits[0].varsI[INDEX_KRAIT_DAMAGE] = 0;

// make copies
for (int ship=1; ship < MAX_KRAITS; ship++)
    {
    memcpy(&kraits[ship], &kraits[0], sizeof(BOB));
    } // end for krait

} // end Init_Aradas

//////////////////////////////////////////////////////////////////////////////////////

void Reset_Kraits(void)
{
// this function resets all the kraits

for (int ship=0; ship < MAX_KRAITS; ship++)
    {
    kraits[ship].state                       = KRAIT_STATE_DEAD;
    kraits[ship].varsI[INDEX_KRAIT_DIR]    = 0;
    kraits[ship].varsI[INDEX_KRAIT_DAMAGE] = 0;
    } // end for krait

} // end Reset_Kraits

///////////////////////////////////////////////////////////

void Start_Krait(int override=0, int x=0, int y=0, int dir=0)
{
// this functions starts an krait, note that if override = 1
// then the function uses the sent data otherwise it's random

// first find an available krait
for (int index=0; index < MAX_KRAITS; index++)
	{
	// is this one dead
	if (kraits[index].state  == KRAIT_STATE_DEAD)
		{
		// position the krait on a circle ring randomly from player

		int ang = rand()%16;
		float xpos = KRAIT_RANGE_RING*cos_look16[ang];
		float ypos = KRAIT_RANGE_RING*sin_look16[ang];

        // set position
		kraits[index].varsI[INDEX_WORLD_X] = player_x+xpos;
		kraits[index].varsI[INDEX_WORLD_Y] = player_y+ypos;

		// set direction to right
		kraits[index].varsI[INDEX_KRAIT_DIR] = ang;

		// set velocity
		kraits[index].xv = 4+rand()%8;
		kraits[index].yv = RAND_RANGE(-4,4);

		
		// set remaining state variables
		kraits[index].state  = KRAIT_STATE_ALIVE;
        kraits[index].varsI[INDEX_KRAIT_DIR] = 0;
        kraits[index].varsI[INDEX_KRAIT_DAMAGE] = 0;

		// done so exit
		return;

		} // end if
	
	} // end for index

} // end Start_Krait

////////////////////////////////////////////////////////////

void Move_Kraits(void)
{
// this function moves all the kraits and does the ai

for (int index=0; index < MAX_KRAITS; index++)
    {
    // test if its alive
    if (kraits[index].state == KRAIT_STATE_ALIVE)
        {
        // move the krait
        kraits[index].varsI[INDEX_WORLD_X]+=kraits[index].xv;
        kraits[index].varsI[INDEX_WORLD_Y]+=kraits[index].yv;
          
        // test for boundaries
        if (kraits[index].varsI[INDEX_WORLD_X] > UNIVERSE_MAX_X)
           {
  		   kraits[index].varsI[INDEX_WORLD_X] = UNIVERSE_MIN_X;
           } // end if
		else
		if (kraits[index].varsI[INDEX_WORLD_X] < UNIVERSE_MIN_X )
           {
  		   kraits[index].varsI[INDEX_WORLD_X] = UNIVERSE_MAX_X;
           } // end if	    

		if (kraits[index].varsI[INDEX_WORLD_Y] > UNIVERSE_MAX_Y)
           {
		   kraits[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MIN_Y;
           } // end if
		else
		if (kraits[index].varsI[INDEX_WORLD_Y] < UNIVERSE_MIN_Y ) 
           {
		   kraits[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MAX_Y;
           } // end if


       // leave particle trail
    if (kraits[index].varsI[INDEX_KRAIT_DAMAGE] > (MAX_KRAIT_DAMAGE >> 2) && 
       (rand()%3)==1)
        {
        Start_Particle(PARTICLE_TYPE_FLICKER, PARTICLE_COLOR_WHITE, 30+rand()%25, 
                      kraits[index].varsI[INDEX_WORLD_X]+RAND_RANGE(-4,4), 
                      kraits[index].varsI[INDEX_WORLD_Y]+RAND_RANGE(-4,4), 
                      (kraits[index].xv*.125), (kraits[index].yv*.125));
    
        Start_Particle(PARTICLE_TYPE_FADE, PARTICLE_COLOR_RED, 5, 
                      kraits[index].varsI[INDEX_WORLD_X]+RAND_RANGE(-4,4), 
                      kraits[index].varsI[INDEX_WORLD_Y]+RAND_RANGE(-4,4), 
                      (kraits[index].xv*.125), (kraits[index].yv*.125));


    
        } // end if

		// do ai
		
		// tracking algorithm

        // compute vector toward player
        float vx = (player_x - cos_look16[crescent.varsI[CRESCENT_INDEX_DIR]]*200 ) - kraits[index].varsI[INDEX_WORLD_X];
        float vy = (player_y - sin_look16[crescent.varsI[CRESCENT_INDEX_DIR]]*200 )- kraits[index].varsI[INDEX_WORLD_Y];

        // normalize vector (sorta :)
        float length = Fast_Distance_2D(vx,vy);


        // only track if reasonable close
        if ((length < MIN_KRAIT_TRACKING_DIST))
            {
            vx=krait_tracking_rate*vx/length;
            vy=krait_tracking_rate*vy/length;

            // add velocity vector to current velocity
            kraits[index].xv+=vx;
            kraits[index].yv+=vy;

            // add a little noise
            if ((rand()%10)==1)
                {
                vx = RAND_RANGE(-1,1);
                vy = RAND_RANGE(-1,1);
                kraits[index].xv+=vx;
                kraits[index].yv+=vy;
                } // end if

            // test velocity vector of kraits
            length = Fast_Distance_2D(kraits[index].xv, kraits[index].yv);

            // test for velocity overflow and slow
            if (length > MAX_KRAIT_VELOCITY)
                {
                // slow down
                kraits[index].xv*=0.75;
                kraits[index].yv*=0.75;

                } // end if

           
        else
            {
            // add a random velocity component
            if ((rand()%30)==1)
                {
                vx = RAND_RANGE(-2,2);
                vy = RAND_RANGE(-2,2);

                // add velocity vector to current velocity
                kraits[index].xv+=vx;
                kraits[index].yv+=vy;

                // test velocity vector of kraits
                length = Fast_Distance_2D(kraits[index].xv, kraits[index].yv);

                // test for velocity overflow and slow
                if (length > MAX_KRAIT_VELOCITY)
                    {
                    // slow down
                    kraits[index].xv*=0.75;
                    kraits[index].yv*=0.75;

                    } // end if
        
                } // end if
           
            } // end else



            // turn turret always
            // compute d0, d1, d2   

            // first create a vector point in the direction of the turret
            
            // compute current turrent vector
            int tdir1 = kraits[index].varsI[INDEX_KRAIT_DIR];
            
            float d1x = kraits[index].varsI[INDEX_WORLD_X] + cos_look16[tdir1]*32;
            float d1y = kraits[index].varsI[INDEX_WORLD_Y] + sin_look16[tdir1]*32;

            // compute turret vector plus one
            int tdir2 = kraits[index].varsI[INDEX_KRAIT_DIR]+1;
            
            if (tdir2 > 15) 
                tdir2 = 0;

            float d2x = kraits[index].varsI[INDEX_WORLD_X] + cos_look16[tdir2]*32;
            float d2y = kraits[index].varsI[INDEX_WORLD_Y] + sin_look16[tdir2]*32;
            
            // compute turret vector minus one
            int tdir0 = kraits[index].varsI[INDEX_KRAIT_DIR]-1;
            
            if (tdir0 < 0) 
                tdir0=15;

            float d0x = kraits[index].varsI[INDEX_WORLD_X] + cos_look16[tdir0]*32;
            float d0y = kraits[index].varsI[INDEX_WORLD_Y] + sin_look16[tdir0]*32;

            // now find the min dist
            float dist0 = Fast_Distance_2D(player_x - d0x, player_y - d0y);
            float dist1 = Fast_Distance_2D(player_x - d1x, player_y - d1y);
            float dist2 = Fast_Distance_2D(player_x - d2x, player_y - d2y);

            if (dist0 < dist2 && dist0 < dist1)
                {
                // the negative direction is best        
                kraits[index].varsI[INDEX_KRAIT_DIR] = tdir0;

                } // end if
            else
            if (dist2 < dist0 && dist2 < dist1)
                {
                // the positive direction is best
                kraits[index].varsI[INDEX_KRAIT_DIR] = tdir2;

                } // end if

            // test for firing
            if ( (rand()%10)==1 &&
				(Fast_Distance_2D(player_x-kraits[index].varsI[INDEX_WORLD_X],
			                 player_y-kraits[index].varsI[INDEX_WORLD_Y]) < KRAIT_MIN_ATTACK_RANGE) )
                {   
					int plasma_x = kraits[index].varsI[INDEX_WORLD_X];
                    int plasma_y = kraits[index].varsI[INDEX_WORLD_Y];


                    float plasma_xv = kraits[index].xv+cos_look16[kraits[index].varsI[INDEX_KRAIT_DIR]]*PLASMA_SPEED_KRAIT;
	                float plasma_yv = kraits[index].yv+sin_look16[kraits[index].varsI[INDEX_KRAIT_DIR]]*PLASMA_SPEED_KRAIT;

                    // fire weapon
                    Fire_Plasma(plasma_x, plasma_y, plasma_xv,plasma_yv,PLASMA_ANIM_ENEMY);

				} // end if fire

	
			
		}	//if close
    } // end if alive
	} // end for index
} // end Move_Kraits

///////////////////////////////////////////////////////////

void Draw_Kraits(void)
{
// this function moves all the kraits

for (int index=0; index < MAX_KRAITS; index++)
    {
    // test if plasma pulse is in flight
    if (kraits[index].state == KRAIT_STATE_ALIVE)
        {
        // transform to screen coords
        kraits[index].x = kraits[index].varsI[INDEX_WORLD_X] - (kraits[index].width >> 1) - player_x + (SCREEN_WIDTH/2);
        kraits[index].y = kraits[index].varsI[INDEX_WORLD_Y] - (kraits[index].height >> 1) - player_y + (SCREEN_HEIGHT/2);

        // select the proper frame
		kraits[index].curr_frame = (kraits[index].varsI[INDEX_KRAIT_DIR] + 16*(rand()%2) );

        // draw the krait
        Draw_BOB(&kraits[index],lpddsback);
         
        } // end if

    } // end for index

} // end Draw_Kraits

////////////////////////////////////////////////////////////

void Delete_Kraits(void)
{
// this function simply deletes all memory and surfaces
// related to the kraits

for (int index=0; index < MAX_KRAITS; index++)
    Destroy_BOB(&kraits[index]);

} // end Delete_Lazira

/////////////////////////////////////////////////////
