
///////////////////////////////////////////////////////////

void Init_Laziras(void)
{
// this function loads and initializes the laziras to a known state
	
int frame;  // looping var

// create the first bob
Create_BOB(&laziras[0],0,0,64,64,32,
            BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME,
            DDSCAPS_SYSTEMMEMORY);

	// load animation frames 
	Load_Bitmap_File(&bitmap8bit, "ART/lazira.bmp");

	for (frame=0; frame < 31; frame++)
		Load_Frame_BOB(&laziras[0],&bitmap8bit,frame,frame%8,frame/8,BITMAP_EXTRACT_MODE_CELL);  

    // unload data infile
    Unload_Bitmap_File(&bitmap8bit);


// set state to off
laziras[0].state                       = LAZIRA_STATE_DEAD;
laziras[0].varsI[INDEX_LAZIRA_DIR]    = 0;
laziras[0].varsI[INDEX_LAZIRA_DAMAGE] = 0;

// make copies
for (int ship=1; ship < MAX_LAZIRAS; ship++)
    {
    memcpy(&laziras[ship], &laziras[0], sizeof(BOB));
    } // end for lazira

} // end Init_Aradas

//////////////////////////////////////////////////////////////////////////////////////

void Reset_Laziras(void)
{
// this function resets all the laziras

for (int ship=0; ship < MAX_LAZIRAS; ship++)
    {
    laziras[ship].state                       = LAZIRA_STATE_DEAD;
    laziras[ship].varsI[INDEX_LAZIRA_DIR]    = 0;
    laziras[ship].varsI[INDEX_LAZIRA_DAMAGE] = 0;
    } // end for lazira

} // end Reset_Laziras

///////////////////////////////////////////////////////////

void Start_Lazira(int override=0, int x=0, int y=0, int dir=0)
{
// this functions starts an lazira, note that if override = 1
// then the function uses the sent data otherwise it's random

// first find an available lazira
for (int index=0; index < MAX_LAZIRAS; index++)
	{
	// is this one dead
	if (laziras[index].state  == LAZIRA_STATE_DEAD)
		{
		// position the lazira on a circle ring randomly from player

		int ang = rand()%16;
		float xpos = LAZIRA_RANGE_RING*cos_look16[ang];
		float ypos = LAZIRA_RANGE_RING*sin_look16[ang];

        // set position
		laziras[index].varsI[INDEX_WORLD_X] = player_x+xpos;
		laziras[index].varsI[INDEX_WORLD_Y] = player_y+ypos;

		// set direction to right
		laziras[index].varsI[INDEX_LAZIRA_DIR] = ang;

		// set velocity
		laziras[index].xv = 4+rand()%8;
		laziras[index].yv = RAND_RANGE(-4,4);

		
		// set remaining state variables
		laziras[index].state  = LAZIRA_STATE_ALIVE;
        laziras[index].varsI[INDEX_LAZIRA_DIR] = 0;
        laziras[index].varsI[INDEX_LAZIRA_DAMAGE] = 0;

		// done so exit
		return;

		} // end if
	
	} // end for index

} // end Start_Lazira

////////////////////////////////////////////////////////////

void Move_Laziras(void)
{
// this function moves all the laziras and does the ai

for (int index=0; index < MAX_LAZIRAS; index++)
    {
    // test if plasma pulse is in flight
    if (laziras[index].state == LAZIRA_STATE_ALIVE)
        {
        // move the lazira
        laziras[index].varsI[INDEX_WORLD_X]+=laziras[index].xv;
        laziras[index].varsI[INDEX_WORLD_Y]+=laziras[index].yv;
          
        // test for boundaries
        if (laziras[index].varsI[INDEX_WORLD_X] > UNIVERSE_MAX_X)
           {
  		   laziras[index].varsI[INDEX_WORLD_X] = UNIVERSE_MIN_X;
           } // end if
		else
		if (laziras[index].varsI[INDEX_WORLD_X] < UNIVERSE_MIN_X )
           {
  		   laziras[index].varsI[INDEX_WORLD_X] = UNIVERSE_MAX_X;
           } // end if	    

		if (laziras[index].varsI[INDEX_WORLD_Y] > UNIVERSE_MAX_Y)
           {
		   laziras[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MIN_Y;
           } // end if
		else
		if (laziras[index].varsI[INDEX_WORLD_Y] < UNIVERSE_MIN_Y ) 
           {
		   laziras[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MAX_Y;
           } // end if


       // leave particle trail
    if (laziras[index].varsI[INDEX_LAZIRA_DAMAGE] > (MAX_LAZIRA_DAMAGE >> 2) && 
       (rand()%3)==1)
        {
        Start_Particle(PARTICLE_TYPE_FLICKER, PARTICLE_COLOR_WHITE, 30+rand()%25, 
                      laziras[index].varsI[INDEX_WORLD_X]+RAND_RANGE(-4,4), 
                      laziras[index].varsI[INDEX_WORLD_Y]+RAND_RANGE(-4,4), 
                      (laziras[index].xv*.125), (laziras[index].yv*.125));
    
        Start_Particle(PARTICLE_TYPE_FADE, PARTICLE_COLOR_RED, 5, 
                      laziras[index].varsI[INDEX_WORLD_X]+RAND_RANGE(-4,4), 
                      laziras[index].varsI[INDEX_WORLD_Y]+RAND_RANGE(-4,4), 
                      (laziras[index].xv*.125), (laziras[index].yv*.125));


    
        } // end if

		// do ai
		
		// tracking algorithm

        // compute vector toward player
        float vx = player_x - laziras[index].varsI[INDEX_WORLD_X];
        float vy = player_y - laziras[index].varsI[INDEX_WORLD_Y];

        // normalize vector (sorta :)
        float length = Fast_Distance_2D(vx,vy);


        // only track if reasonable close
        if (length < MIN_LAZIRA_TRACKING_DIST)
            {
            vx=lazira_tracking_rate*vx/length;
            vy=lazira_tracking_rate*vy/length;

            // add velocity vector to current velocity
            laziras[index].xv+=vx;
            laziras[index].yv+=vy;

            // add a little noise
            if ((rand()%10)==1)
                {
                vx = RAND_RANGE(-1,1);
                vy = RAND_RANGE(-1,1);
                laziras[index].xv+=vx;
                laziras[index].yv+=vy;
                } // end if

            // test velocity vector of laziras
            length = Fast_Distance_2D(laziras[index].xv, laziras[index].yv);

            // test for velocity overflow and slow
            if (length > MAX_LAZIRA_VELOCITY)
                {
                // slow down
                laziras[index].xv*=0.75;
                laziras[index].yv*=0.75;

                } // end if


        else
            {
            // add a random velocity component
            if ((rand()%30)==1)
                {
                vx = RAND_RANGE(-2,2);
                vy = RAND_RANGE(-2,2);

                // add velocity vector to current velocity
                laziras[index].xv+=vx;
                laziras[index].yv+=vy;

                // test velocity vector of laziras
                length = Fast_Distance_2D(laziras[index].xv, laziras[index].yv);

                // test for velocity overflow and slow
                if (length > MAX_LAZIRA_VELOCITY)
                    {
                    // slow down
                    laziras[index].xv*=0.75;
                    laziras[index].yv*=0.75;

                    } // end if
        
                } // end if
           
            } // end else

        } // end if activated
    else
        {
        // compute dist from player
        float vx = player_x - laziras[index].varsI[INDEX_WORLD_X];
        float vy = player_y - laziras[index].varsI[INDEX_WORLD_Y];

        float length = Fast_Distance_2D(vx,vy);
		}

            // turn turret always
            // compute d0, d1, d2   

            // first create a vector point in the direction of the turret
            
            // compute current turrent vector
            int tdir1 = laziras[index].varsI[INDEX_LAZIRA_DIR];
            
            float d1x = laziras[index].varsI[INDEX_WORLD_X] + cos_look16[tdir1]*32;
            float d1y = laziras[index].varsI[INDEX_WORLD_Y] + sin_look16[tdir1]*32;

            // compute turret vector plus one
            int tdir2 = laziras[index].varsI[INDEX_LAZIRA_DIR]+1;
            
            if (tdir2 > 15) 
                tdir2 = 0;

            float d2x = laziras[index].varsI[INDEX_WORLD_X] + cos_look16[tdir2]*32;
            float d2y = laziras[index].varsI[INDEX_WORLD_Y] + sin_look16[tdir2]*32;
            
            // compute turret vector minus one
            int tdir0 = laziras[index].varsI[INDEX_LAZIRA_DIR]-1;
            
            if (tdir0 < 0) 
                tdir0=15;

            float d0x = laziras[index].varsI[INDEX_WORLD_X] + cos_look16[tdir0]*32;
            float d0y = laziras[index].varsI[INDEX_WORLD_Y] + sin_look16[tdir0]*32;

            // now find the min dist
            float dist0 = Fast_Distance_2D(player_x - d0x, player_y - d0y);
            float dist1 = Fast_Distance_2D(player_x - d1x, player_y - d1y);
            float dist2 = Fast_Distance_2D(player_x - d2x, player_y - d2y);

            if (dist0 < dist2 && dist0 < dist1)
                {
                // the negative direction is best        
                laziras[index].varsI[INDEX_LAZIRA_DIR] = tdir0;

                } // end if
            else
            if (dist2 < dist0 && dist2 < dist1)
                {
                // the positive direction is best
                laziras[index].varsI[INDEX_LAZIRA_DIR] = tdir2;

                } // end if

            // test for firing
            if ( (rand()%25)==1 &&
				(Fast_Distance_2D(player_x-laziras[index].varsI[INDEX_WORLD_X],
			                 player_y-laziras[index].varsI[INDEX_WORLD_Y]) < LAZIRA_MIN_ATTACK_RANGE) )
                {   
					int plasma_x = laziras[index].varsI[INDEX_WORLD_X];
                    int plasma_y = laziras[index].varsI[INDEX_WORLD_Y];


                    float plasma_xv = laziras[index].xv+cos_look16[laziras[index].varsI[INDEX_LAZIRA_DIR]]*PLASMA_SPEED_LAZIRA;
	                float plasma_yv = laziras[index].yv+sin_look16[laziras[index].varsI[INDEX_LAZIRA_DIR]]*PLASMA_SPEED_LAZIRA;

                    // fire weapon
                    Fire_Plasma(plasma_x, plasma_y, plasma_xv,plasma_yv,PLASMA_ANIM_ENEMY);

				} // end if fire

			//} // end if

		} // end if alive

	} // end for index

} // end Move_Laziras

///////////////////////////////////////////////////////////

void Draw_Laziras(void)
{
// this function moves all the laziras

for (int index=0; index < MAX_LAZIRAS; index++)
    {
    // test if plasma pulse is in flight
    if (laziras[index].state == LAZIRA_STATE_ALIVE)
        {
        // transform to screen coords
        laziras[index].x = laziras[index].varsI[INDEX_WORLD_X] - (laziras[index].width >> 1) - player_x + (SCREEN_WIDTH/2);
        laziras[index].y = laziras[index].varsI[INDEX_WORLD_Y] - (laziras[index].height >> 1) - player_y + (SCREEN_HEIGHT/2);

        // select the proper frame
		laziras[index].curr_frame = (laziras[index].varsI[INDEX_LAZIRA_DIR] + 16*(rand()%2) );

        // draw the lazira
        Draw_BOB(&laziras[index],lpddsback);
         
        } // end if

    } // end for index

} // end Draw_Laziras

////////////////////////////////////////////////////////////

void Delete_Laziras(void)
{
// this function simply deletes all memory and surfaces
// related to the laziras

for (int index=0; index < MAX_LAZIRAS; index++)
    Destroy_BOB(&laziras[index]);

} // end Delete_Lazira

/////////////////////////////////////////////////////
