void Init_Miranu(void)
{
// this function loads and initializes the miranu to a known state
	
int frame;  // looping var

// create the first bob
Create_BOB(&miranu[0],0,0,120,120,32,
            BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME,
            DDSCAPS_SYSTEMMEMORY);

	for (frame=0; frame <= 31; frame++)
	{
	// load the rocks imagery 
    Pad_Name("ART/MIRANU", "BMP", buffer, frame);
	Load_Bitmap_File(&bitmap8bit, buffer);		

    // load the actual .BMP
    Load_Frame_BOB(&miranu[0],&bitmap8bit,frame,0,0,BITMAP_EXTRACT_MODE_ABS);  

    // unload data infile
    Unload_Bitmap_File(&bitmap8bit);

	} // end if


// set state to off
miranu[0].state                       = MIRANU_STATE_DEAD;
miranu[0].varsI[INDEX_MIRANU_DIR]    = 0;
miranu[0].varsI[INDEX_MIRANU_DAMAGE] = 0;

// make copies
for (int ship=1; ship < MAX_MIRANU; ship++)
    {
    memcpy(&miranu[ship], &miranu[0], sizeof(BOB));
    } // end for zidegar

} // end Init_Aradas

//////////////////////////////////////////////////////////////////////////////////////

void Reset_Miranu(void)
{
// this function resets all the miranu

for (int ship=0; ship < MAX_MIRANU; ship++)
    {
    miranu[ship].state                       = MIRANU_STATE_DEAD;
    miranu[ship].varsI[INDEX_MIRANU_DIR]    = 0;
    miranu[ship].varsI[INDEX_MIRANU_DAMAGE] = 0;
    } // end for zidegar

} // end Reset_Miranu

///////////////////////////////////////////////////////////

void Start_Miranu(int override=0, int x=0, int y=0, int dir=0)
{
// this functions starts a miranu, note that if override = 1
// then the function uses the sent data otherwise it's random

// first find an available zidegar
for (int index=0; index < MAX_MIRANU; index++)
	{
	// is this one dead
	if (miranu[index].state  == MIRANU_STATE_DEAD)
		{
		// position the miranu on a circle ring randomly from player

		int ang = rand()%16;
		float xpos = MIRANU_RANGE_RING*cos_look16[ang];
		float ypos = MIRANU_RANGE_RING*sin_look16[ang];

        // set position
		miranu[index].varsI[INDEX_WORLD_X] = player_x+xpos;
		miranu[index].varsI[INDEX_WORLD_Y] = player_y+ypos;

		// set direction to right
		miranu[index].varsI[INDEX_MIRANU_DIR] = ang;

		// set velocity
		miranu[index].xv = 4+rand()%8;
		miranu[index].yv = RAND_RANGE(-4,4);

		
		// set remaining state variables
		miranu[index].state  = MIRANU_STATE_ALIVE;
        miranu[index].varsI[INDEX_MIRANU_DIR] = 0;
        miranu[index].varsI[INDEX_MIRANU_DAMAGE] = 0;

		// done so exit
		return;

		} // end if
	
	} // end for index

} // end Start_Miranu

////////////////////////////////////////////////////////////

void Move_Miranu(void)
{
// this function moves all the miranu and does the ai

for (int index=0; index < MAX_MIRANU; index++)
    {
    // test if its alive
    if (miranu[index].state == MIRANU_STATE_ALIVE)
        {
        // move the zidegar
        miranu[index].varsI[INDEX_WORLD_X]+=miranu[index].xv;
        miranu[index].varsI[INDEX_WORLD_Y]+=miranu[index].yv;
          
        // test for boundaries
        if (miranu[index].varsI[INDEX_WORLD_X] > UNIVERSE_MAX_X)
           {
  		   miranu[index].varsI[INDEX_WORLD_X] = UNIVERSE_MIN_X;
           } // end if
		else
		if (miranu[index].varsI[INDEX_WORLD_X] < UNIVERSE_MIN_X )
           {
  		   miranu[index].varsI[INDEX_WORLD_X] = UNIVERSE_MAX_X;
           } // end if	    

		if (miranu[index].varsI[INDEX_WORLD_Y] > UNIVERSE_MAX_Y)
           {
		   miranu[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MIN_Y;
           } // end if
		else
		if (miranu[index].varsI[INDEX_WORLD_Y] < UNIVERSE_MIN_Y ) 
           {
		   miranu[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MAX_Y;
           } // end if


       // leave particle trail
    if (miranu[index].varsI[INDEX_MIRANU_DAMAGE] > (MAX_MIRANU_DAMAGE >> 2) && 
       (rand()%3)==1)
        {
        Start_Particle(PARTICLE_TYPE_FLICKER, PARTICLE_COLOR_WHITE, 30+rand()%25, 
                      miranu[index].varsI[INDEX_WORLD_X]+RAND_RANGE(-4,4), 
                      miranu[index].varsI[INDEX_WORLD_Y]+RAND_RANGE(-4,4), 
                      (miranu[index].xv*.125), (miranu[index].yv*.125));
    
        Start_Particle(PARTICLE_TYPE_FADE, PARTICLE_COLOR_RED, 5, 
                      miranu[index].varsI[INDEX_WORLD_X]+RAND_RANGE(-4,4), 
                      miranu[index].varsI[INDEX_WORLD_Y]+RAND_RANGE(-4,4), 
                      (miranu[index].xv*.125), (miranu[index].yv*.125));


    
        } // end if

		// do ai
		
		// tracking algorithm

        // compute vector toward player
        float vx = (player_x - cos_look16[crescent.varsI[CRESCENT_INDEX_DIR]]*200 ) - miranu[index].varsI[INDEX_WORLD_X];
        float vy = (player_y - sin_look16[crescent.varsI[CRESCENT_INDEX_DIR]]*200 )- miranu[index].varsI[INDEX_WORLD_Y];

        // normalize vector (sorta :)
        float length = Fast_Distance_2D(vx,vy);


        // only track if reasonable close
        if ((length < MIN_MIRANU_TRACKING_DIST))
            {
            vx=zidegar_tracking_rate*vx/length;
            vy=zidegar_tracking_rate*vy/length;

            // add velocity vector to current velocity
            miranu[index].xv+=vx;
            miranu[index].yv+=vy;

            // add a little noise
            if ((rand()%10)==1)
                {
                vx = RAND_RANGE(-1,1);
                vy = RAND_RANGE(-1,1);
                miranu[index].xv+=vx;
                miranu[index].yv+=vy;
                } // end if

            // test velocity vector of miranu
            length = Fast_Distance_2D(miranu[index].xv, miranu[index].yv);

            // test for velocity overflow and slow
            if (length > MAX_MIRANU_VELOCITY)
                {
                // slow down
                miranu[index].xv*=0.75;
                miranu[index].yv*=0.75;

                } // end if

           
        else
            {
            // add a random velocity component
            if ((rand()%30)==1)
                {
                vx = RAND_RANGE(-2,2);
                vy = RAND_RANGE(-2,2);

                // add velocity vector to current velocity
                miranu[index].xv+=vx;
                miranu[index].yv+=vy;

                // test velocity vector of miranu
                length = Fast_Distance_2D(miranu[index].xv, miranu[index].yv);

                // test for velocity overflow and slow
                if (length > MAX_MIRANU_VELOCITY)
                    {
                    // slow down
                    miranu[index].xv*=0.75;
                    miranu[index].yv*=0.75;

                    } // end if
        
                } // end if
           
            } // end else



            // turn turret always
            // compute d0, d1, d2   

            // first create a vector point in the direction of the turret
            
            // compute current turrent vector
            int tdir1 = miranu[index].varsI[INDEX_MIRANU_DIR];
            
            float d1x = miranu[index].varsI[INDEX_WORLD_X] + cos_look16[tdir1]*32;
            float d1y = miranu[index].varsI[INDEX_WORLD_Y] + sin_look16[tdir1]*32;

            // compute turret vector plus one
            int tdir2 = miranu[index].varsI[INDEX_MIRANU_DIR]+1;
            
            if (tdir2 > 15) 
                tdir2 = 0;

            float d2x = miranu[index].varsI[INDEX_WORLD_X] + cos_look16[tdir2]*32;
            float d2y = miranu[index].varsI[INDEX_WORLD_Y] + sin_look16[tdir2]*32;
            
            // compute turret vector minus one
            int tdir0 = miranu[index].varsI[INDEX_MIRANU_DIR]-1;
            
            if (tdir0 < 0) 
                tdir0=15;

            float d0x = miranu[index].varsI[INDEX_WORLD_X] + cos_look16[tdir0]*32;
            float d0y = miranu[index].varsI[INDEX_WORLD_Y] + sin_look16[tdir0]*32;

            // now find the min dist
            float dist0 = Fast_Distance_2D(player_x - d0x, player_y - d0y);
            float dist1 = Fast_Distance_2D(player_x - d1x, player_y - d1y);
            float dist2 = Fast_Distance_2D(player_x - d2x, player_y - d2y);

            if (dist0 < dist2 && dist0 < dist1)
                {
                // the negative direction is best        
                miranu[index].varsI[INDEX_MIRANU_DIR] = tdir0;

                } // end if
            else
            if (dist2 < dist0 && dist2 < dist1)
                {
                // the positive direction is best
                miranu[index].varsI[INDEX_MIRANU_DIR] = tdir2;

                } // end if

            // test for firing
            if ( (rand()%5)==1 &&
				(Fast_Distance_2D(player_x-miranu[index].varsI[INDEX_WORLD_X],
			                 player_y-miranu[index].varsI[INDEX_WORLD_Y]) < MIRANU_MIN_ATTACK_RANGE) )
                {   
					int plasma_x = miranu[index].varsI[INDEX_WORLD_X];
                    int plasma_y = miranu[index].varsI[INDEX_WORLD_Y];


                    float plasma_xv = miranu[index].xv+cos_look16[miranu[index].varsI[INDEX_MIRANU_DIR]]*PLASMA_SPEED_MIRANU;
	                float plasma_yv = miranu[index].yv+sin_look16[miranu[index].varsI[INDEX_MIRANU_DIR]]*PLASMA_SPEED_MIRANU;

                    // fire weapon
                    Fire_Plasma(plasma_x, plasma_y, plasma_xv,plasma_yv,PLASMA_ANIM_ENEMY);

				} // end if fire

	
		}	//if close
    } // end if alive
	} // end for index
} // end Move_Miranu

///////////////////////////////////////////////////////////

void Draw_Miranu(void)
{
// this function moves all the miranu

for (int index=0; index < MAX_MIRANU; index++)
    {
    // test if plasma pulse is in flight
    if (miranu[index].state == MIRANU_STATE_ALIVE)
        {
        // transform to screen coords
        miranu[index].x = miranu[index].varsI[INDEX_WORLD_X] - (miranu[index].width >> 1) - player_x + (SCREEN_WIDTH/2);
        miranu[index].y = miranu[index].varsI[INDEX_WORLD_Y] - (miranu[index].height >> 1) - player_y + (SCREEN_HEIGHT/2);

        // select the proper frame
		miranu[index].curr_frame = (miranu[index].varsI[INDEX_MIRANU_DIR] + 16*(rand()%2) );

        // draw the miranu
        Draw_BOB(&miranu[index],lpddsback);
         
        } // end if

    } // end for index

} // end Draw_Miranu

////////////////////////////////////////////////////////////

void Delete_Miranu(void)
{
// this function simply deletes all memory and surfaces
// related to the miranu

for (int index=0; index < MAX_MIRANU; index++)
    Destroy_BOB(&miranu[index]);

} // end Delete_Miranu

/////////////////////////////////////////////////////
