
///////////////////////////////////////////////////////////

void Init_Rocks(void)
{
// this function initializes and loads all the rocks 

int frame; // used as loop index

// create the large rock
Create_BOB(&rock_l,0,0,96,96,16,
            BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME,
            DDSCAPS_SYSTEMMEMORY);

// load animation frames
for (frame=0; frame <= 15; frame++)
	{
	// load the rocks imagery 
    Pad_Name("ART/ROCKL", "BMP", buffer, frame);
	Load_Bitmap_File(&bitmap8bit, buffer);		

    // load the actual .BMP
    Load_Frame_BOB(&rock_l,&bitmap8bit,frame,0,0,BITMAP_EXTRACT_MODE_ABS);  

    // unload data infile
    Unload_Bitmap_File(&bitmap8bit);

	} // end if

// set animation rate
Set_Anim_Speed_BOB(&rock_l,1+rand()%5);
Set_Vel_BOB(&rock_l, -4+rand()%8, 4+rand()%4);
Set_Pos_BOB(&rock_l, rand()%SCREEN_WIDTH, rand()%SCREEN_HEIGHT);

// set size of rock
rock_l.varsI[0] = ROCK_LARGE;
rock_l.varsI[1] = rock_sizes[rock_l.varsI[0]];

// set state to off
rock_l.state = ROCK_STATE_OFF;

// create the medium rock
Create_BOB(&rock_m,0,0,56,56,16,
            BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME,
            DDSCAPS_SYSTEMMEMORY);

// load animation frames
for (frame=0; frame <= 15; frame++)
	{
	// load the rocks imagery 
    Pad_Name("ART/ROCKM", "BMP", buffer, frame);
	Load_Bitmap_File(&bitmap8bit, buffer);		

    // load the actual .BMP
    Load_Frame_BOB(&rock_m,&bitmap8bit,frame,0,0,BITMAP_EXTRACT_MODE_ABS);  

    // unload data infile
    Unload_Bitmap_File(&bitmap8bit);

	} // end if

// set animation rate
Set_Anim_Speed_BOB(&rock_m,1+rand()%5);
Set_Vel_BOB(&rock_m, -4+rand()%8, 4+rand()%4);
Set_Pos_BOB(&rock_m, rand()%SCREEN_WIDTH, rand()%SCREEN_HEIGHT);

// set size of rock
rock_m.varsI[0] = ROCK_MEDIUM;
rock_m.varsI[1] = rock_sizes[rock_m.varsI[0]];

// set state to off
rock_m.state = ROCK_STATE_OFF;

// create the small rock
Create_BOB(&rock_s,0,0,32,32,16,
            BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME,
            DDSCAPS_SYSTEMMEMORY);

// load animation frames
for (frame=0; frame <= 15; frame++)
	{
	// load the rocks imagery 
    Pad_Name("ART/ROCKS", "BMP", buffer, frame);
	Load_Bitmap_File(&bitmap8bit, buffer);		

    // load the actual .BMP
    Load_Frame_BOB(&rock_s,&bitmap8bit,frame,0,0,BITMAP_EXTRACT_MODE_ABS);  

    // unload data infile
    Unload_Bitmap_File(&bitmap8bit);

	} // end if

// set animation rate
Set_Anim_Speed_BOB(&rock_s,1+rand()%5);
Set_Vel_BOB(&rock_s, -4+rand()%8, -4+rand()%8);
Set_Pos_BOB(&rock_s, rand()%SCREEN_WIDTH, rand()%SCREEN_HEIGHT);

// set size of rock
rock_s.varsI[0] = ROCK_SMALL;
rock_s.varsI[1] = rock_sizes[rock_s.varsI[0]];

// set state to off
rock_s.state = ROCK_STATE_OFF;

// make semi copies
for (int rock=0; rock < MAX_ROCKS; rock++)
    {
    memcpy(&rocks[rock], &rock_l, sizeof(BOB));

    // set animation rate
    Set_Anim_Speed_BOB(&rocks[rock],1+rand()%5);
    
    // set velocity
    Set_Vel_BOB(&rocks[rock], -4+rand()%8, -4+rand()%8);
    
    // set position
    Set_Pos_BOB(&rocks[rock],0,0);
    rocks[rock].varsI[INDEX_WORLD_X] = RAND_RANGE(UNIVERSE_MIN_X, UNIVERSE_MAX_X);
    rocks[rock].varsI[INDEX_WORLD_Y] = RAND_RANGE(UNIVERSE_MIN_Y, UNIVERSE_MAX_Y);    

    // set size of rock
    rocks[rock].varsI[0] = rand()%3;
    rocks[rock].varsI[1] = rock_sizes[rocks[rock].varsI[0]];

    // based on size of rock switch in correct bitmaps and sizes
	switch(rocks[rock].varsI[1])
		{
		case ROCK_LARGE:		
			{
            // copy dd bitmap surfaces
            memcpy(rocks[rock].images,rock_l.images,sizeof(LPDIRECTDRAWSURFACE4)*MAX_BOB_FRAMES);

			// set width and height (incase)
			rocks[rock].width  = rock_l.width;
			rocks[rock].height = rock_l.height;

			} break;

		case ROCK_MEDIUM:
			{
            // copy dd bitmap surfaces
            memcpy(rocks[rock].images,rock_m.images,sizeof(LPDIRECTDRAWSURFACE4)*MAX_BOB_FRAMES);

			// set width and height (incase)
			rocks[rock].width  = rock_m.width;
			rocks[rock].height = rock_m.height;

			} break;

		case ROCK_SMALL:
			{
            // copy dd bitmap surfaces
            memcpy(rocks[rock].images,rock_s.images,sizeof(LPDIRECTDRAWSURFACE4)*MAX_BOB_FRAMES);

			// set width and height (incase)
			rocks[rock].width  = rock_s.width;
			rocks[rock].height = rock_s.height;

			} break;
		
		default: break;

		} // end switch
   
    } // end for rock

} // end Init_Rocks

//////////////////////////////////////////////////////////////////////////////

void Reset_Rocks(void)
{
// this function resets all the rocks

for (int rock=0; rock < MAX_ROCKS; rock++)
    {
    // reset state
    rocks[rock].state = ROCK_STATE_OFF;

    } // end for rocks

} // end Reset_Rocks

//////////////////////////////////////////////////////////////////////////////

void Delete_Rocks(void)
{
// this function simply deletes all memory and surfaces
// related to the rocks pulses

for (int index=0; index < MAX_ROCKS; index++)
    Destroy_BOB(&rocks[index]);

} // end Delete_Rocks

//////////////////////////////////////////////////////////////////////////////

void Seed_Rocks(void)
{
// this function seeds the universe with rocks

int index; // looping var

   // scan for a rock to initialize
   for (index=0; index < (MAX_ROCKS*.75); index++)
       {
       // is this rock available?
       if (rocks[index].state == ROCK_STATE_OFF)
          {
          // set animation rate
          Set_Anim_Speed_BOB(&rocks[index],1+rand()%5);
    
          // set velocity
          Set_Vel_BOB(&rocks[index], -6+rand()%12, -6+rand()%12);
    
          // set position
          Set_Pos_BOB(&rocks[index],0,0);
		  rocks[index].varsI[INDEX_WORLD_X] = RAND_RANGE(UNIVERSE_MIN_X, UNIVERSE_MAX_X);
		  rocks[index].varsI[INDEX_WORLD_Y] = RAND_RANGE(UNIVERSE_MIN_Y, UNIVERSE_MAX_Y);    
    
          // set size of rock
          rocks[index].varsI[0] = rand()%3;
          rocks[index].varsI[1] = rock_sizes[rocks[index].varsI[0]];
          
          // set strength of rock
          switch(rocks[index].varsI[0])
                {
                case ROCK_LARGE:
                     {
                     // set hardness of rock
                     rocks[index].varsI[2] = 100+rand()%100;

					 // copy dd bitmap surfaces
					 memcpy(rocks[index].images,rock_l.images,sizeof(LPDIRECTDRAWSURFACE4)*MAX_BOB_FRAMES);


		  			 // set width and height (incase)
		 			 rocks[index].width  = rock_l.width;
					 rocks[index].height = rock_l.height;

                     } break;
 
                case ROCK_MEDIUM:
                     {
                     // set hardness of rock
                     rocks[index].varsI[2] = 40 + rand()%30;

					 // copy dd bitmap surfaces
					 memcpy(rocks[index].images,rock_m.images,sizeof(LPDIRECTDRAWSURFACE4)*MAX_BOB_FRAMES);


					 // set width and height (incase)
					 rocks[index].width  = rock_m.width;
				     rocks[index].height = rock_m.height;

                     } break;     

                case ROCK_SMALL:
                     {
                     // set hardness of rock
                     rocks[index].varsI[2] = 10;

					 // copy dd bitmap surfaces
		             memcpy(rocks[index].images,rock_s.images,sizeof(LPDIRECTDRAWSURFACE4)*MAX_BOB_FRAMES);

					 // set width and height (incase)
					 rocks[index].width  = rock_s.width;
					 rocks[index].height = rock_s.height;
                     } break;

                default:break;

                } // end switch

          // turn rock on
          rocks[index].state = ROCK_STATE_ON;

          } // end if

       } // end for index

} // end Seed_Rocks


//////////////////////////////////////////////////////////

void Move_Rocks(void)
{
// this function moves all the rocks pulses and checks for
// collision with the rocks

for (int index=0; index < MAX_ROCKS; index++)
    {
    // test if rocks pulse is in flight
    if (rocks[index].state == ROCK_STATE_ON)
        {
        // move the rock
        rocks[index].varsI[INDEX_WORLD_X]+=rocks[index].xv;
        rocks[index].varsI[INDEX_WORLD_Y]+=rocks[index].yv;

        // test if rock is out of universe, but persist object
        if (rocks[index].varsI[INDEX_WORLD_X] > UNIVERSE_MAX_X)
            rocks[index].varsI[INDEX_WORLD_X] = UNIVERSE_MIN_X;
        else
        if (rocks[index].varsI[INDEX_WORLD_X] < UNIVERSE_MIN_X)
            rocks[index].varsI[INDEX_WORLD_X] = UNIVERSE_MAX_X;

        if (rocks[index].varsI[INDEX_WORLD_Y] > UNIVERSE_MAX_X)
            rocks[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MIN_X;
        else
        if (rocks[index].varsI[INDEX_WORLD_Y] < UNIVERSE_MIN_Y)
            rocks[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MAX_Y;
    

        // test for collision with rocks
        if (player_state == PLAYER_STATE_ALIVE && Collision_Test(player_x-(crescent.width*.5), player_y-(crescent.height*.5), 
                           crescent.width, crescent.height,
                           rocks[index].varsI[INDEX_WORLD_X]-(rocks[index].width*.5), 
						   rocks[index].varsI[INDEX_WORLD_Y]-(rocks[index].height*.5),
                           rocks[index].width, rocks[index].height))
        {
        // what size rock did we hit?
        switch(rocks[index].varsI[0])
              {
              case ROCK_LARGE:
                   {
                   // start explosion
                   Start_Burst(rocks[index].varsI[INDEX_WORLD_X], rocks[index].varsI[INDEX_WORLD_Y], 
                               68+rand()%12,54+rand()%10,
                               rocks[index].xv*.5, rocks[index].yv*.5);
                   
                    // update players damage
                    player_damage+=35;

					// update velocity vector
                    player_xv+=(rocks[index].xv);
					player_yv+=(rocks[index].yv);

                    // rotate ship a bit
					crescent.varsI[CRESCENT_INDEX_DIR]+=(RAND_RANGE(-4,4));
                    if (crescent.varsI[CRESCENT_INDEX_DIR] > 15)
						crescent.varsI[CRESCENT_INDEX_DIR] -= 15;
					else
                    if (crescent.varsI[CRESCENT_INDEX_DIR] < 0)
						crescent.varsI[CRESCENT_INDEX_DIR] += 15;


                    } break;

               case ROCK_MEDIUM:
                    {
                    // start explosion
                    Start_Burst(rocks[index].varsI[INDEX_WORLD_X], rocks[index].varsI[INDEX_WORLD_Y], 
                                52+rand()%10,44+rand()%8,
                                rocks[index].xv*.5, rocks[index].yv*.5);

                    // update players damage
                    player_damage+=15;

					// update velocity vector
                    player_xv+=(rocks[index].xv*.5);
					player_yv+=(rocks[index].yv*.5);

                    // rotate ship a bit
					crescent.varsI[CRESCENT_INDEX_DIR]+=(RAND_RANGE(-4,4));
                    if (crescent.varsI[CRESCENT_INDEX_DIR] > 15)
						crescent.varsI[CRESCENT_INDEX_DIR] -= 15;
					else
                    if (crescent.varsI[CRESCENT_INDEX_DIR] < 0)
						crescent.varsI[CRESCENT_INDEX_DIR] += 15;

                    } break;

                case ROCK_SMALL:
                     {
                     // start explosion
                     Start_Burst(rocks[index].varsI[INDEX_WORLD_X], rocks[index].varsI[INDEX_WORLD_Y], 
                                 34-4+rand()%8,30-3+rand()%6,
                                 rocks[index].xv*.5, rocks[index].yv*.5);

                      // update players damage
                      player_damage+=5;

  					  // update velocity vector
                      player_xv+=(rocks[index].xv*.25);
 					  player_yv+=(rocks[index].yv*.25);

                    // rotate ship a bit
					crescent.varsI[CRESCENT_INDEX_DIR]+=(RAND_RANGE(-4,4));
                    if (crescent.varsI[CRESCENT_INDEX_DIR] > 15)
						crescent.varsI[CRESCENT_INDEX_DIR] -= 15;
					else
                    if (crescent.varsI[CRESCENT_INDEX_DIR] < 0)
						crescent.varsI[CRESCENT_INDEX_DIR] += 15;


                      } break;
                          
                } // end switch

                // update score
                player_score+=rocks[index].varsI[2];

                // engage shields
                player_shield_count=3;

                // kill the original
                rocks[index].state = ROCK_STATE_OFF;     

            } // end if collision

		} // end if rock

    } // end for index

// now test if it's time to add a new rock to the list
if ((rand()%100)==50)
   {
   // scan for a rock to initialize
   for (index=0; index < MAX_ROCKS; index++)
       {
       // is this rock available?
       if (rocks[index].state == ROCK_STATE_OFF)
          {
          // set animation rate
          Set_Anim_Speed_BOB(&rocks[index],1+rand()%5);
    
          // set velocity
          Set_Vel_BOB(&rocks[index], -6+rand()%12, -6+rand()%12);
    
          // set position
          Set_Pos_BOB(&rocks[index],0,0);
		  rocks[index].varsI[INDEX_WORLD_X] = RAND_RANGE(UNIVERSE_MIN_X, UNIVERSE_MAX_X);
		  rocks[index].varsI[INDEX_WORLD_Y] = RAND_RANGE(UNIVERSE_MIN_Y, UNIVERSE_MAX_Y);    
    
          // set size of rock
          rocks[index].varsI[0] = rand()%3;
          rocks[index].varsI[1] = rock_sizes[rocks[index].varsI[0]];
          
          // set strength of rock
          switch(rocks[index].varsI[0])
                {
                case ROCK_LARGE:
                     {
                     // set hardness of rock
                     rocks[index].varsI[2] = 100+rand()%100;

					 // copy dd bitmap surfaces
					 memcpy(rocks[index].images,rock_l.images,sizeof(LPDIRECTDRAWSURFACE4)*MAX_BOB_FRAMES);


		  			 // set width and height (incase)
		 			 rocks[index].width  = rock_l.width;
					 rocks[index].height = rock_l.height;

                     } break;
 
                case ROCK_MEDIUM:
                     {
                     // set hardness of rock
                     rocks[index].varsI[2] = 40 + rand()%30;

					 // copy dd bitmap surfaces
					 memcpy(rocks[index].images,rock_m.images,sizeof(LPDIRECTDRAWSURFACE4)*MAX_BOB_FRAMES);


					 // set width and height (incase)
					 rocks[index].width  = rock_m.width;
				     rocks[index].height = rock_m.height;

                     } break;     

                case ROCK_SMALL:
                     {
                     // set hardness of rock
                     rocks[index].varsI[2] = 10;

					 // copy dd bitmap surfaces
		             memcpy(rocks[index].images,rock_s.images,sizeof(LPDIRECTDRAWSURFACE4)*MAX_BOB_FRAMES);

					 // set width and height (incase)
					 rocks[index].width  = rock_s.width;
					 rocks[index].height = rock_s.height;
                     } break;

                default:break;

                } // end switch


          // turn rock on
          rocks[index].state = ROCK_STATE_ON;
          
          // later
          return;

          } // end if

       } // end for index

   } // end if

} // end Move_Rocks

///////////////////////////////////////////////////////////

void Start_Rock(int x, int y, int size,int xv, int yv)
{
// this functions starts a rock up with the sent parms, considering
// there is one left

// scan for a rock to initialize
   for (int index=0; index < MAX_ROCKS; index++)
       {
       // is this rock available?
       if (rocks[index].state == ROCK_STATE_OFF)
          {
          // set animation rate
          Set_Anim_Speed_BOB(&rocks[index],1+rand()%5);
    
          // set velocity
          Set_Vel_BOB(&rocks[index], xv,yv);
    
          // set position
          Set_Pos_BOB(&rocks[index], 0,0);
          rocks[index].varsI[INDEX_WORLD_X] = x;
          rocks[index].varsI[INDEX_WORLD_Y] = y;
    
          // set size of rock
          rocks[index].varsI[0] = size;
          rocks[index].varsI[1] = rock_sizes[rocks[index].varsI[0]];
          
          // set strength of rock
          switch(rocks[index].varsI[0])
                {
                case ROCK_LARGE:
                     {
                     // set hardness of rock
                     rocks[index].varsI[2] = 100+rand()%100;

			 	     // copy dd bitmap surfaces
		             memcpy(rocks[index].images,rock_l.images,sizeof(LPDIRECTDRAWSURFACE4)*MAX_BOB_FRAMES);

					 // set width and height (incase)
					 rocks[index].width  = rock_l.width;
					 rocks[index].height = rock_l.height;

                     } break;
 
                case ROCK_MEDIUM:
                     {
                     // set hardness of rock
                     rocks[index].varsI[2] = 40 + rand()%30;

					 // copy dd bitmap surfaces
					 memcpy(rocks[index].images,rock_m.images,sizeof(LPDIRECTDRAWSURFACE4)*MAX_BOB_FRAMES);

					 // set width and height (incase)
					 rocks[index].width  = rock_m.width;
					 rocks[index].height = rock_m.height;
                     } break;     

                case ROCK_SMALL:
                     {
                     // set hardness of rock
                     rocks[index].varsI[2] = 10;

			         // copy dd bitmap surfaces
					 memcpy(rocks[index].images,rock_s.images,sizeof(LPDIRECTDRAWSURFACE4)*MAX_BOB_FRAMES);

					 // set width and height (incase)
					 rocks[index].width  = rock_s.width;
					 rocks[index].height = rock_s.height;
                     } break;

                default:break;

                } // end switch

          // turn rock on
          rocks[index].state = ROCK_STATE_ON;
          
          // later
          return;

          } // end if

       } // end for index

} // end Start_Rock

///////////////////////////////////////////////////////////

void Draw_Rocks(void)
{
// this function draws all the rocks 

for (int index=0; index<MAX_ROCKS; index++)
    {
    // test if rocks pulse is in flight
    if (rocks[index].state == ROCK_STATE_ON)
        {
        // transform to screen coords
        rocks[index].x = rocks[index].varsI[INDEX_WORLD_X] - (rocks[index].width*.5) - player_x + (SCREEN_WIDTH/2);
        rocks[index].y = rocks[index].varsI[INDEX_WORLD_Y] - (rocks[index].height*.5) - player_y + (SCREEN_HEIGHT/2);

        // draw the rock
        Draw_BOB(&rocks[index],lpddsback);
         
        // animate the pulse
        Animate_BOB(&rocks[index]);

        } // end if

    } // end for index

} // end Draw_Rocks
