// OUTPOST.CPP - OutPost Game Demo

#include "includes and defines.h"
#include "arada.h"
#include "lazira.h"
#include "krait.h"
#include "zidegar.h"
#include "miranu.h"
#include "extra funcs.h"
#include "plasma.h"
#include "particle.h"
#include "rock.h"


// FUNCTIONS //////////////////////////////////////////////

LRESULT CALLBACK WindowProc(HWND hwnd, 
						    UINT msg, 
                            WPARAM wparam, 
                            LPARAM lparam)
{
// this is the main message handler of the system
PAINTSTRUCT	ps;		   // used in WM_PAINT
HDC			hdc;	   // handle to a device context

// what is the message 
switch(msg)
	{	
	case WM_CREATE: 
        {
		// do initialization stuff here


		return(0);
		} break;

    case WM_PAINT:
         {
         // start painting
         hdc = BeginPaint(hwnd,&ps);

         // end painting
         EndPaint(hwnd,&ps);
         return(0);
        } break;


	case WM_DESTROY: 
		{

		// kill the application			
		PostQuitMessage(0);
		return(0);
		} break;

	default:break;

    } // end switch

// process any messages that we didn't take care of 
return (DefWindowProc(hwnd, msg, wparam, lparam));

} // end WinProc

// WINMAIN ////////////////////////////////////////////////

int WINAPI WinMain(	HINSTANCE hinstance,
					HINSTANCE hprevinstance,
					LPSTR lpcmdline,
					int ncmdshow)
{
// this is the winmain function

WNDCLASS winclass;	// this will hold the class we create
HWND	 hwnd;		// generic window handle
MSG		 msg;		// generic message
//HDC      hdc;       // generic dc
//PAINTSTRUCT ps;     // generic paintstruct

// first fill in the window class stucture
winclass.style			= CS_DBLCLKS | CS_OWNDC | 
                          CS_HREDRAW | CS_VREDRAW;
winclass.lpfnWndProc	= WindowProc;
winclass.cbClsExtra		= 0;
winclass.cbWndExtra		= 0;
winclass.hInstance		= hinstance;
winclass.hIcon			= LoadIcon(NULL, IDI_APPLICATION);
winclass.hCursor		= LoadCursor(NULL, IDC_ARROW);
winclass.hbrBackground	= (HBRUSH)GetStockObject(BLACK_BRUSH);
winclass.lpszMenuName	= NULL; 
winclass.lpszClassName	= WINDOW_CLASS_NAME;

// register the window class
if (!RegisterClass(&winclass))
	return(0);

// create the window, note the use of WS_POPUP
if (!(hwnd = CreateWindow(WINDOW_CLASS_NAME, // class
						  "Star Escape",	 // title
						  WS_POPUP | WS_VISIBLE,
					 	  0,0,	   // x,y
						  WINDOW_WIDTH,  // width
                          WINDOW_HEIGHT, // height
						  NULL,	   // handle to parent 
						  NULL,	   // handle to menu
						  hinstance,// instance
						  NULL)))	// creation parms
return(0);

// save the window handle and instance in a global
main_window_handle = hwnd;
main_instance      = hinstance;

// perform all game console specific initialization
Game_Init();

// enter main event loop
while(1)
	{
	if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
		{ 
		// test if this is a quit
        if (msg.message == WM_QUIT)
           break;
	
		// translate any accelerator keys
		TranslateMessage(&msg);

		// send the message to the window proc
		DispatchMessage(&msg);
		} // end if
    
    // main game processing goes here
    Game_Main();

	} // end while


// shutdown game and release all resources
Game_Shutdown();

// return to Windows like this
return(msg.wParam);

} // end WinMain

/////////////////////////////////////////////////////////////////

void Init_Stars(void)
{
// this function initializes all the stars in the star field

int index; // looping variable

for (index=0; index < MAX_STARS; index++)
    {

    // select plane that star will be in

    switch(rand()%3)
          {
          case STAR_PLANE_0:
               {
               stars[index].color = STAR_COLOR_0;
               stars[index].plane = STAR_PLANE_0;
               } break;

          case STAR_PLANE_1:
               {
               stars[index].color = STAR_COLOR_1;
               stars[index].plane = STAR_PLANE_1;
               } break;

          case STAR_PLANE_2:
               {
               stars[index].color = STAR_COLOR_2;
               stars[index].plane = STAR_PLANE_2;
               } break;

          default:break;

          } // end switch

     // set fields that aren't plane specific
     stars[index].x = rand()%SCREEN_WIDTH;   // change this latter to reflect clipping
     stars[index].y = rand()%SCREEN_HEIGHT;  // region

    } // end for index

} // end Init_Stars

/////////////////////////////////////////////////////////////////////////////

void Move_Stars(void)
{
// this function moves the star field, note that the star field is always
// in screen coordinates, otherwise, we would need thousands of stars to
// fill up the universe instead of 50!

int index,   // looping variable
    star_x,  // used as fast aliases to star position
    star_y,
    plane_0_dx,
    plane_0_dy,
    plane_1_dx,
    plane_1_dy,
    plane_2_dx,
    plane_2_dy;

// pre-compute plane translations
plane_0_dx = -int(player_xv) >> 2;
plane_0_dy = -int(player_yv) >> 2;

plane_1_dx = -int(player_xv) >> 1;
plane_1_dy = -int(player_yv) >> 1;

plane_2_dx = -int(player_xv);
plane_2_dy = -int(player_yv);

// move all the stars based on the motion of the player

for (index=0; index < MAX_STARS; index++)
    {
    // locally cache star position to speed up calculations
    star_x = stars[index].x;
    star_y = stars[index].y;

    // test which star field star is in so it is translated with correct
    // perspective

    switch(stars[index].plane)
          {
          case STAR_PLANE_0:
               {
               // move the star based on differntial motion of player
               // far plane is divided by 4

               star_x+=plane_0_dx;
               star_y+=plane_0_dy;

               } break;

          case STAR_PLANE_1:
               {
               // move the star based on differntial motion of player
               // middle plane is divided by 2

               star_x+=plane_1_dx;
               star_y+=plane_1_dy;

               } break;

          case STAR_PLANE_2:
               {
               // move the star based on differntial motion of player
               // near plane is divided by 1

               star_x+=plane_2_dx;
               star_y+=plane_2_dy;

               } break;

          } // end switch plane

    // test if a star has flown off an edge

    if (star_x >= SCREEN_WIDTH)
        star_x = star_x-SCREEN_WIDTH;
    else
    if (star_x < 0)
        star_x = SCREEN_WIDTH + star_x;

    if (star_y >= SCREEN_HEIGHT)
        star_y = star_y-SCREEN_HEIGHT;
    else
    if (star_y < 0)
        star_y = SCREEN_HEIGHT+star_y;

    // reset stars position in structure
    stars[index].x = star_x;
    stars[index].y = star_y;

    } // end for index

} // end Move_Stars

/////////////////////////////////////////////////////////////////////////////

void Draw_Stars(void)
{
// this function draws all the stars

// lock back surface
DDraw_Lock_Back_Surface();

// draw all the stars
for (int index=0; index < MAX_STARS; index++)
    {
    // draw stars 
    Draw_Pixel(stars[index].x,stars[index].y, stars[index].color,back_buffer, back_lpitch);

    } // end for index

// unlock the secondary surface
DDraw_Unlock_Back_Surface();

} // end Draw_Stars

///////////////////////////////////////////////////

void Draw_Scanner(void)
{
// this function draws the scanner

int index,sx,sy; // looping and position

// lock back surface
DDraw_Lock_Back_Surface();

// draw all the rocks
for (index=0; index < MAX_ROCKS; index++)
    {
    // draw rock blips
	if (rocks[index].state==ROCK_STATE_ON)
		{
		sx = ((rocks[index].varsI[INDEX_WORLD_X] - UNIVERSE_MIN_X) >> 7) + (SCREEN_WIDTH/2) - ((UNIVERSE_MAX_X - UNIVERSE_MIN_X) >> 8);
		sy = ((rocks[index].varsI[INDEX_WORLD_Y] - UNIVERSE_MIN_Y) >> 7) + 32;
	    
		Draw_Pixel(sx,sy,8,back_buffer, back_lpitch);
		} // end if

    } // end for index

for (index=0; index < MAX_ARADAS; index++)
    {
    // draw arada blips
	if (aradas[index].state==ARADA_STATE_ALIVE)
		{
		sx = ((aradas[index].varsI[INDEX_WORLD_X] - UNIVERSE_MIN_X) >> 7) + (SCREEN_WIDTH/2) - ((UNIVERSE_MAX_X - UNIVERSE_MIN_X) >> 8);
		sy = ((aradas[index].varsI[INDEX_WORLD_Y] - UNIVERSE_MIN_Y) >> 7) + 32;
	    
		Draw_Pixel(sx,sy,12,back_buffer, back_lpitch);

		} // end if

    } // end for index

for (index=0; index < MAX_LAZIRAS; index++)
    {
    // draw lazira blips
	if (laziras[index].state==LAZIRA_STATE_ALIVE)
		{
		sx = ((laziras[index].varsI[INDEX_WORLD_X] - UNIVERSE_MIN_X) >> 7) + (SCREEN_WIDTH/2) - ((UNIVERSE_MAX_X - UNIVERSE_MIN_X) >> 8);
		sy = ((laziras[index].varsI[INDEX_WORLD_Y] - UNIVERSE_MIN_Y) >> 7) + 32;
	    
		Draw_Pixel(sx,sy,12,back_buffer, back_lpitch);

		} // end if

    } // end for index

for (index=0; index < MAX_ZIDEGARS; index++)
    {
    // draw zidegar blips
	if (zidegars[index].state==ZIDEGAR_STATE_ALIVE)
		{
		sx = ((zidegars[index].varsI[INDEX_WORLD_X] - UNIVERSE_MIN_X) >> 7) + (SCREEN_WIDTH/2) - ((UNIVERSE_MAX_X - UNIVERSE_MIN_X) >> 8);
		sy = ((zidegars[index].varsI[INDEX_WORLD_Y] - UNIVERSE_MIN_Y) >> 7) + 32;
	    
		Draw_Pixel(sx,sy,12,back_buffer, back_lpitch);

		} // end if

} // end for index
for (index=0; index < MAX_KRAITS; index++)
    {
    // draw krait blips
	if (kraits[index].state==KRAIT_STATE_ALIVE)
		{
		sx = ((kraits[index].varsI[INDEX_WORLD_X] - UNIVERSE_MIN_X) >> 7) + (SCREEN_WIDTH/2) - ((UNIVERSE_MAX_X - UNIVERSE_MIN_X) >> 8);
		sy = ((kraits[index].varsI[INDEX_WORLD_Y] - UNIVERSE_MIN_Y) >> 7) + 32;
	    
		Draw_Pixel(sx,sy,12,back_buffer, back_lpitch);

		} // end if

} // end for index

for (index=0; index < MAX_MIRANU; index++)
    {
    // draw miranu blips
	if (miranu[index].state==MIRANU_STATE_ALIVE)
		{
		sx = ((miranu[index].varsI[INDEX_WORLD_X] - UNIVERSE_MIN_X) >> 7) + (SCREEN_WIDTH/2) - ((UNIVERSE_MAX_X - UNIVERSE_MIN_X) >> 8);
		sy = ((miranu[index].varsI[INDEX_WORLD_Y] - UNIVERSE_MIN_Y) >> 7) + 32;
	    
		Draw_Pixel(sx,sy,12,back_buffer, back_lpitch);
		Draw_Pixel(sx+1,sy,12,back_buffer, back_lpitch);
		Draw_Pixel(sx,sy+1,12,back_buffer, back_lpitch);
		Draw_Pixel(sx+1,sy+1,12,back_buffer, back_lpitch);

		} // end if

} // end for index


// unlock the secondary surface
DDraw_Unlock_Back_Surface();

// draw all the stations
for (index=0; index < MAX_STATIONS; index++)
    {
    // draw station blips
	if (stations[index].state==STATION_STATE_ALIVE)
		{
		sx = ((stations[index].varsI[INDEX_WORLD_X] - UNIVERSE_MIN_X) >> 7) + (SCREEN_WIDTH/2) - ((UNIVERSE_MAX_X - UNIVERSE_MIN_X) >> 8);
		sy = ((stations[index].varsI[INDEX_WORLD_Y] - UNIVERSE_MIN_Y) >> 7) + 32;
	    
        // test for state
		if (stations[index].anim_state == STATION_SHIELDS_ANIM_ON)
			{
            stationsmall.curr_frame = 0;
            stationsmall.x = sx - 3;
            stationsmall.y = sy - 3; 
            Draw_BOB(&stationsmall,lpddsback);            


			} // end if
		else
			{

            stationsmall.curr_frame = 1;
            stationsmall.x = sx - 3;
            stationsmall.y = sy - 3; 
            Draw_BOB(&stationsmall,lpddsback);  


			} // end if

		} // end if

    } // end for index


// unlock the secondary surface
DDraw_Lock_Back_Surface();

// draw player as white blip
sx = ((int(player_x) - UNIVERSE_MIN_X) >> 7) + (SCREEN_WIDTH/2) - ((UNIVERSE_MAX_X - UNIVERSE_MIN_X) >> 8);
sy = ((int(player_y) - UNIVERSE_MIN_Y) >> 7) + 32;

int col = rand()%256;

Draw_Pixel(sx,sy,col,back_buffer, back_lpitch);
Draw_Pixel(sx+1,sy,col,back_buffer, back_lpitch);
Draw_Pixel(sx,sy+1,col,back_buffer, back_lpitch);
Draw_Pixel(sx+1,sy+1,col,back_buffer, back_lpitch);

// unlock the secondary surface
DDraw_Unlock_Back_Surface();

// now draw the art around the edges

hud.x          = 320-64;
hud.y          = 32-4;
hud.curr_frame = 0;
Draw_BOB(&hud,lpddsback);

hud.x          = 320+64-16;
hud.y          = 32-4;
hud.curr_frame = 1;
Draw_BOB(&hud,lpddsback);

hud.x          = 320-64;
hud.y          = 32+128-20;
hud.curr_frame = 2;
Draw_BOB(&hud,lpddsback);


hud.x          = 320+64-16;
hud.y          = 32+128-20;
hud.curr_frame = 3;
Draw_BOB(&hud,lpddsback);


} // end Draw_Scanner


///////////////////////////////////////////////////////////

void Init_Stations(void)
{
// this function loads and initializes the stations to a known state
	
static int shields_on_anim[16] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15},
           shields_off_anim[1] = {16};

int frame;  // looping va

// create the first bob
Create_BOB(&stations[0],0,0,192,144,17,
            BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_ANIM,
            DDSCAPS_SYSTEMMEMORY);

// load animation frames 
for (frame=0; frame <= 16; frame++)
	{
	// load the rocks imagery 
    Pad_Name("ART/STATION", "BMP", buffer, frame);
	Load_Bitmap_File(&bitmap8bit, buffer);		

    // load the actual .BMP
    Load_Frame_BOB(&stations[0],&bitmap8bit,frame,0,0,BITMAP_EXTRACT_MODE_ABS);  

    // unload data infile
    Unload_Bitmap_File(&bitmap8bit);

	} // end if

// set state to off
stations[0].state  = STATION_STATE_ALIVE;

// set anim state
stations[0].anim_state = STATION_SHIELDS_ANIM_ON;

// set damage to 0
stations[0].varsI[INDEX_STATION_DAMAGE] = 0;

// set animation rate
Set_Anim_Speed_BOB(&stations[0],15);

// load in the shield on/off animations
Load_Animation_BOB(&stations[0], STATION_SHIELDS_ANIM_ON, 16, shields_on_anim);
Load_Animation_BOB(&stations[0], STATION_SHIELDS_ANIM_OFF, 1, shields_off_anim);

// set animation to on
Set_Animation_BOB(&stations[0], STATION_SHIELDS_ANIM_ON);

// make copies
for (int ship=1; ship < MAX_STATIONS; ship++)
    {
    memcpy(&stations[ship], &stations[0], sizeof(BOB));
    } // end for pulse


    // load the miniature station
    Create_BOB(&stationsmall,0,0,8,8,2,
               BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME,
               DDSCAPS_SYSTEMMEMORY);

    Load_Bitmap_File(&bitmap8bit, "ART/STATIONSMALL8.BMP");		

    // load the actual .BMP
    Load_Frame_BOB(&stationsmall,&bitmap8bit,0,0,0,BITMAP_EXTRACT_MODE_CELL);  
    Load_Frame_BOB(&stationsmall,&bitmap8bit,1,1,0,BITMAP_EXTRACT_MODE_CELL);  

    // unload data infile
    Unload_Bitmap_File(&bitmap8bit);


} // end Init_Stations

///////////////////////////////////////////////////////////

void Reset_Stations(void)
{
// this function resets all the stations in preparation for another run

for (int index=0; index < MAX_STATIONS; index++)
	{
    // set state to off
    stations[index].state  = STATION_STATE_DEAD;

    // set anim state
    stations[index].anim_state = STATION_SHIELDS_ANIM_ON;

    // set damage to 0
    stations[index].varsI[INDEX_STATION_DAMAGE] = 0;

    } // end for

} // end Reset_Stations

///////////////////////////////////////////////////////////

void Start_Station(int override=0, int x=0, int y=0)
{
// this functions starts a station, note that if override = 1
// then the function uses the sent data otherwise it's random

// first find an available stations
for (int index=0; index < MAX_STATIONS; index++)
	{
	// is this one dead
	if (stations[index].state  == STATION_STATE_DEAD)
		{
		// position the station

		int xpos = RAND_RANGE((UNIVERSE_MIN_X+256),(UNIVERSE_MAX_X-256));
		int ypos = RAND_RANGE((UNIVERSE_MIN_Y+256),(UNIVERSE_MAX_Y-256));

        // set position
		stations[index].varsI[INDEX_WORLD_X] = xpos;
		stations[index].varsI[INDEX_WORLD_Y] = ypos;

  	    // set velocity
		stations[index].xv = 0;
		stations[index].yv = 0;

		// set remaining state variables
		stations[index].state  = STATION_STATE_ALIVE;

		// set animation to on
        Set_Animation_BOB(&stations[index], STATION_SHIELDS_ANIM_ON);

		// set anim state
        stations[index].anim_state = STATION_SHIELDS_ANIM_ON;

        // set damage to 0
        stations[index].varsI[INDEX_STATION_DAMAGE] = 0;

		// done so exit
		return;

		} // end if
	
	} // end for index

} // end Start_Station

////////////////////////////////////////////////////////////

void Move_Stations(void)
{
// this function moves/animates all the stations

for (int index=0; index < MAX_STATIONS; index++)
    {
    // test if station is alive
    if (stations[index].state == STATION_STATE_ALIVE)
        {
        // move the stations
        /*stations[index].varsI[INDEX_WORLD_X]+=stations[index].xv;
        stations[index].varsI[INDEX_WORLD_Y]+=stations[index].yv;
          
        // test for boundaries
        if (stations[index].varsI[INDEX_WORLD_X] > UNIVERSE_MAX_X)
           {
  		   stations[index].varsI[INDEX_WORLD_X] = UNIVERSE_MIN_X;
           } // end if
		else
		if (stations[index].varsI[INDEX_WORLD_X] < UNIVERSE_MIN_X )
           {
  		   stations[index].varsI[INDEX_WORLD_X] = UNIVERSE_MAX_X;
           } // end if	    

		if (stations[index].varsI[INDEX_WORLD_Y] > UNIVERSE_MAX_Y)
           {
		   stations[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MIN_Y;
           } // end if
		else
		if (stations[index].varsI[INDEX_WORLD_Y] < UNIVERSE_MIN_Y ) 
           {
		   stations[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MAX_Y;
           } // end if*/

        // test for damage level
        if (stations[index].varsI[INDEX_STATION_DAMAGE] > (MAX_STATION_DAMAGE/4) &&
            (rand()%(20 - (stations[index].varsI[INDEX_STATION_DAMAGE] >> 3))) == 1)
            {
            int width = 20+rand()%60;

             // start a burst
             Start_Burst(stations[index].varsI[INDEX_WORLD_X] - (stations[index].width*.5)+RAND_RANGE(0,stations[index].width),
                         stations[index].varsI[INDEX_WORLD_Y] - (stations[index].height*.5)+RAND_RANGE(0,stations[index].height), 
                         width,(width >> 2) + (width >> 1),
                         int(stations[index].xv)>>1, int(stations[index].yv)>>1);   

            // add some particles

            } // end if

		} // end if alive

	} // end for index

} // end Move_Stations

///////////////////////////////////////////////////////////

void Draw_Stations(void)
{
// this function draws all the stations

for (int index=0; index < MAX_STATIONS; index++)
    {
    // test if station is alive
    if (stations[index].state == STATION_STATE_ALIVE)
        {
        // transform to screen coords
        stations[index].x = stations[index].varsI[INDEX_WORLD_X] - (stations[index].width >> 1) - player_x + (SCREEN_WIDTH/2);
        stations[index].y = stations[index].varsI[INDEX_WORLD_Y] - (stations[index].height >> 1) - player_y + (SCREEN_HEIGHT/2);

        // draw the station
        Draw_BOB(&stations[index],lpddsback);

		// animate the station
        Animate_BOB(&stations[index]);
         
        } // end if

    } // end for index

} // end Draw_Stations

////////////////////////////////////////////////////////////

void Delete_Stations(void)
{
// this function simply deletes all memory and surfaces
// related to the stations

for (int index=0; index < MAX_STATIONS; index++)
    Destroy_BOB(&stations[index]);

} // end Delete_Stations

///////////////////////////////////////////////////////////

void Init_Powerups(void)
{
// this function loads and initializes the powerups to a known state
	
int index;  // looping var

// create the first bob
Create_BOB(&powerups[0],0,0,32,32,3,
            BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME,
            DDSCAPS_SYSTEMMEMORY);

// load the powerups 
Pad_Name("ART/AMMOPU", "BMP", buffer, 0);
Load_Bitmap_File(&bitmap8bit, buffer);		

// load the actual .BMP
Load_Frame_BOB(&powerups[0],&bitmap8bit,0,0,0,BITMAP_EXTRACT_MODE_ABS);  

// unload data infile
Unload_Bitmap_File(&bitmap8bit);

Pad_Name("ART/SHLDPU", "BMP", buffer, 0);
Load_Bitmap_File(&bitmap8bit, buffer);		

// load the actual .BMP
Load_Frame_BOB(&powerups[0],&bitmap8bit,1,0,0,BITMAP_EXTRACT_MODE_ABS);  

// unload data infile
Unload_Bitmap_File(&bitmap8bit);

Pad_Name("ART/MAPPU", "BMP", buffer, 0);
Load_Bitmap_File(&bitmap8bit, buffer);		

// load the actual .BMP
Load_Frame_BOB(&powerups[0],&bitmap8bit,2,0,0,BITMAP_EXTRACT_MODE_ABS);  

// unload data infile
Unload_Bitmap_File(&bitmap8bit);

// set state to off
powerups[0].state                     = POWERUP_STATE_DEAD;
powerups[0].varsI[INDEX_POWERUP_TYPE] = POWERUP_TYPE_AMMO;

// make copies
for (index=1; index < MAX_POWERUPS; index++)
    {
    Clone_BOB(&powerups[0], &powerups[index]);
    } // end for index

} // end Init_Powerups

//////////////////////////////////////////////////////////////////////////////////////

void Reset_Powerups(void)
{
// this function resets all the powerups

for (int index=0; index < MAX_POWERUPS; index++)
    {
    powerups[index].state                      = POWERUP_STATE_DEAD;
    powerups[index].varsI[INDEX_POWERUP_TYPE]  = POWERUP_TYPE_AMMO;
    } // end for powerups

} // end Reset_Powerups

///////////////////////////////////////////////////////////

void Start_Powerup(int type, int x, int y)
{
// this functions starts a powerup if one is available

// first find an available powerup
for (int index=0; index < MAX_POWERUPS; index++)
	{
	// is this one dead
	if (powerups[index].state == POWERUP_STATE_DEAD)
		{
		// position the powerup

        // set position
		powerups[index].varsI[INDEX_WORLD_X] = x;
		powerups[index].varsI[INDEX_WORLD_Y] = y;

        // set velocity
        powerups[index].xv = RAND_RANGE(-2,2);
        powerups[index].yv = RAND_RANGE(-2,2);

		// set remaining state variables
		powerups[index].state                     = POWERUP_STATE_ALIVE;
        powerups[index].varsI[INDEX_POWERUP_TYPE] = type;

        // set lifespan
        powerups[index].counter_1 = RAND_RANGE(300,600);

		// done so exit
		return;

		} // end if
	
	} // end for index

} // end Start_Powerup

////////////////////////////////////////////////////////////

void Move_Powerups(void)
{
// this function moves all the powerups and does the ai

for (int index=0; index < MAX_POWERUPS; index++)
    {
    // test if powerup is alive
    if (powerups[index].state == POWERUP_STATE_ALIVE)
        {
        // check for collision with player
        if (Collision_Test(player_x-(crescent.width*.5), 
                           player_y-(crescent.height*.5), 
                            crescent.width, crescent.height,
                            powerups[index].varsI[INDEX_WORLD_X]-(powerups[index].width*.5), 
			     		    powerups[index].varsI[INDEX_WORLD_Y]-(powerups[index].height*.5),
                            powerups[index].width, powerups[index].height)) 
             {
             // do the powerup
             if (powerups[index].varsI[INDEX_POWERUP_TYPE] == POWERUP_TYPE_AMMO)
                {
                // increase ammo
                if(WeaponUpgradeLevel<3)
					WeaponUpgradeLevel++;
                // make sound
                DSound_Play(ammopu_id);
                } // end if
             else
             if (powerups[index].varsI[INDEX_POWERUP_TYPE] == POWERUP_TYPE_SHLD)
                {
                // increase shield
                if ((player_damage-=10) < 0)
                     player_damage = 0;

                // make sound
                DSound_Play(shldpu_id);

                } // end if

				else
             if (powerups[index].varsI[INDEX_POWERUP_TYPE] == POWERUP_TYPE_MAP)
                {
					Send_Message("I found a map. Now I can get out of here!!", PLAYER_FACE);
					GotMap = 1;
                // make sound
                DSound_Play(mappu_id);

                } // end if
      
             // kill powerup
             powerups[index].state = POWERUP_STATE_DEAD;
             } // end if

        // move the powerups
        powerups[index].varsI[INDEX_WORLD_X]+=powerups[index].xv;
        powerups[index].varsI[INDEX_WORLD_Y]+=powerups[index].yv;

        // test if time to terminate
        if (--powerups[index].counter_1 < 0)
           {
           // kill powerup
           powerups[index].state = POWERUP_STATE_DEAD;
           } // end if
           
        // test for boundaries
        if (powerups[index].varsI[INDEX_WORLD_X] > UNIVERSE_MAX_X)
           {
  		   powerups[index].varsI[INDEX_WORLD_X] = UNIVERSE_MIN_X;
           } // end if
		else
		if (powerups[index].varsI[INDEX_WORLD_X] < UNIVERSE_MIN_X )
           {
  		   powerups[index].varsI[INDEX_WORLD_X] = UNIVERSE_MAX_X;
           } // end if	    

		if (powerups[index].varsI[INDEX_WORLD_Y] > UNIVERSE_MAX_Y)
           {
		   powerups[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MIN_Y;
           } // end if
		else
		if (powerups[index].varsI[INDEX_WORLD_Y] < UNIVERSE_MIN_Y ) 
           {
		   powerups[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MAX_Y;
           } // end if

		} // end if alive

	} // end for index

} // end Move_Powerups

///////////////////////////////////////////////////////////

void Draw_Powerups(void)
{
// this function draws all the Powerups

for (int index=0; index < MAX_POWERUPS; index++)
    {
    // test if Powerups pulse is aluive
    if (powerups[index].state == POWERUP_STATE_ALIVE)
        {
        // transform to screen coords
        powerups[index].x = powerups[index].varsI[INDEX_WORLD_X] - (powerups[index].width >> 1) - player_x + (SCREEN_WIDTH/2);
        powerups[index].y = powerups[index].varsI[INDEX_WORLD_Y] - (powerups[index].height >> 1) - player_y + (SCREEN_HEIGHT/2);

        // select the proper frame
		powerups[index].curr_frame = powerups[index].varsI[INDEX_POWERUP_TYPE];

        // draw the Powerups
        Draw_BOB(&powerups[index],lpddsback);
         
        } // end if

    } // end for index

} // end Draw_Powerups

////////////////////////////////////////////////////////////

void Delete_Powerups(void)
{
// this function simply deletes all memory and surfaces
// related to the Powerups

for (int index=0; index < MAX_POWERUPS; index++)
    Destroy_BOB(&powerups[index]);

} // end Delete_Powerups

//////////////////////////////////////////////////////


///////////////////////////////////////////////////////////

int Pad_Name(char *filename, char *extension, char *padstring, int num)
{
//int index;
char buffer[80];

// build up blank padstring
sprintf(padstring, "%s0000.%s", filename, extension);

// this function pads a string with 0's
itoa(num, buffer, 10);

// compute the position of the last digit
int last_digit = strlen(filename)+4-1;

// now copy the number into the padstring at the correct position
memcpy(padstring + last_digit - strlen(buffer) + 1, buffer, strlen(buffer));

// return success
return(1);

} // end Pad_Name

///////////////////////////////////////////////////////////

void Init_Bursts(void)
{
// this function initializes and loads all the bursts 

// load the bursts imagery 
Load_Bitmap_File(&bitmap8bit, "ART/EXPL8.BMP");

// create the first bob
Create_BOB(&bursts[0],0,0,42,36,14,
            BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME,
            DDSCAPS_SYSTEMMEMORY);

// load animation frames
for (int frame=0; frame < 14; frame++)
     Load_Frame_BOB(&bursts[0],&bitmap8bit,frame,frame%6,frame/6,BITMAP_EXTRACT_MODE_CELL);  

// set animation rate
Set_Anim_Speed_BOB(&bursts[0],1);

// set size of burst
bursts[0].varsI[0] = bursts[0].width;
bursts[0].varsI[1] = bursts[0].height;

// set state to off
bursts[0].state = BURST_STATE_OFF;

for (int burst=1; burst<MAX_BURSTS; burst++)
    {
    memcpy(&bursts[burst], &bursts[0], sizeof(BOB));
    } // end for burst

// unload data infile
Unload_Bitmap_File(&bitmap8bit);

} // end Init_Bursts

///////////////////////////////////////////////////////////

void Reset_Bursts(void)
{
// this function resets all the bursts

for (int burst=0; burst < MAX_BURSTS; burst++)
    {
    // set state to off
    bursts[burst].state = BURST_STATE_OFF;
    } // end for burst


} // end Reset_Bursts


////////////////////////////////////////////////////////////

void Delete_Bursts(void)
{
// this function simply deletes all memory and surfaces
// related to the bursts pulses

for (int index=0; index < MAX_BURSTS; index++)
    Destroy_BOB(&bursts[index]);

} // end Delete_Bursts

///////////////////////////////////////////////////////////

void Move_Bursts(void)
{
// this function moves all the bursts 

for (int index=0; index < MAX_BURSTS; index++)
    {
    // test if bursts pulse is in moving
    if (bursts[index].state == BURST_STATE_ON)
       {
       // move the burst
       // Move_BOB(&bursts[index]);
       bursts[index].varsI[INDEX_WORLD_X]+=bursts[index].xv;
       bursts[index].varsI[INDEX_WORLD_Y]+=bursts[index].yv;

        // test if burst is off screen or done with animation
        if (bursts[index].curr_frame >= bursts[index].num_frames-1) 
           {
           // kill burst and put back on available list
           bursts[index].state = BURST_STATE_OFF;
           } // end if
        
        } // end if

    } // end for index

} // end Move_Bursts

///////////////////////////////////////////////////////////

void Draw_Bursts(void)
{
// this function draws all the bursts 
for (int index=0; index < MAX_BURSTS; index++)
    {
    // test if bursts pulse is in flight
    if (bursts[index].state == BURST_STATE_ON)
        {
		// transform bursts to screen space
        bursts[index].x = bursts[index].varsI[INDEX_WORLD_X] - (bursts[index].width*.5) - player_x + (SCREEN_WIDTH/2);
        bursts[index].y = bursts[index].varsI[INDEX_WORLD_Y] - (bursts[index].height*.5) - player_y + (SCREEN_HEIGHT/2);

		// is scaling needed
        if (bursts[index].varsI[0]!=bursts[index].width || 
            bursts[index].varsI[1]!=bursts[index].height)
            {
            // draw the burst scaled
            Draw_Scaled_BOB(&bursts[index],
                            bursts[index].varsI[0],bursts[index].varsI[1],
                            lpddsback);
            }
        else // draw normal
           Draw_BOB(&bursts[index],lpddsback);
         
        // animate the explosion
        Animate_BOB(&bursts[index]);

        } // end if

    } // end for index

} // end Draw_Bursts

///////////////////////////////////////////////////////////

void Start_Burst(int x, int y, int width, int height, int xv,int yv)
{
// this function starts a burst up

// now test if it's time to add a new burst to the list

// scan for a burst to initialize
for (int index=0; index < MAX_BURSTS; index++)
    {
    // is this burst available?
    if (bursts[index].state == BURST_STATE_OFF)
       {
       // set animation rate
       Set_Anim_Speed_BOB(&bursts[index],1);
       bursts[index].curr_frame = 0;
    
       // set velocity
       Set_Vel_BOB(&bursts[index], xv,yv);
    
       // set position
       Set_Pos_BOB(&bursts[index], 0,0);
	   bursts[index].varsI[INDEX_WORLD_X] = x;
	   bursts[index].varsI[INDEX_WORLD_Y] = y;
    
       // set size of burst
       bursts[index].varsI[0] = width;
       bursts[index].varsI[1] = height;

       // turn burst on
       bursts[index].state = BURST_STATE_ON;

       // shoot some particles out
       Start_Particle_Explosion(PARTICLE_TYPE_FLICKER, RAND_RANGE(PARTICLE_COLOR_RED, PARTICLE_COLOR_GREEN),20+rand()%40, 
                                bursts[index].varsI[INDEX_WORLD_X], bursts[index].varsI[INDEX_WORLD_Y], 
                                bursts[index].xv, bursts[index].yv, 
                                8+rand()%8); 


       // start sound up
       for (int sound_index=0; sound_index < MAX_EXPL_SOUNDS; sound_index++)
           {
           // test if this sound is playing
           if (DSound_Status_Sound(expl_ids[sound_index])==0)
              {
              DSound_Play(expl_ids[sound_index]);
              break;
              } // end if
        
           } // end for sound_index       

       // later
       return;

       } // end if
 
    } // end for index

} // end Start_Burst

///////////////////////////////////////////////////////////

void Draw_Info(void)
{
// this function draws all the information at the top of the screen

char score[16]; // hold score
char threat[16];
static int red_glow = 0; // used for damage display
static int warning_count = 0;

// build up score string
sprintf(score,"0000000%d",player_score);

// build up final string
sprintf(buffer,"SCORE %s",&score[strlen(score)-8]);
Draw_Text_GDI(buffer,10,10,RGB(0,255,0),lpddsback);

// draw damage
sprintf(buffer,"SHIELDS %d%%",100-player_damage);

if (player_damage < 90)
   Draw_Text_GDI(buffer,350-8*strlen(buffer),10,RGB(0,255,0),lpddsback);
else
  Draw_Text_GDI(buffer,350-8*strlen(buffer),10,RGB(red_glow,0,0),lpddsback);

// update red glow
if ((red_glow+=15) > 255)
   red_glow = 0;

// draw ship info
sprintf(buffer,"SHIPS %d",player_ships);
Draw_Text_GDI(buffer,520,10,RGB(0,255,0),lpddsback);

sprintf(buffer,"VEL %.2f Kps",vel*(100/MAX_PLAYER_SPEED));
Draw_Text_GDI(buffer,10,460,RGB(0,255,0),lpddsback);

sprintf(buffer,"POS [%.2f,%.2f]",player_x, player_y);
Draw_Text_GDI(buffer,480,460,RGB(0,255,0),lpddsback);

} // end Draw_Info


///////////////////////////////////////////////////////////

int Copy_Screen(UCHAR *source_bitmap,UCHAR *dest_buffer, int lpitch, int transparent)
{
// this function draws the bitmap onto the destination memory surface
// if transparent is 1 then color 0 will be transparent
// note this function does NOT clip, so be carefull!!!

UCHAR *dest_addr,   // starting address of bitmap in destination
      *source_addr; // starting adddress of bitmap data in source

UCHAR pixel;        // used to hold pixel value

int index,          // looping vars
    pixel_x;

// test if this bitmap is loaded

   // compute starting destination address
   dest_addr = dest_buffer;

   // compute the starting source address
   source_addr = source_bitmap;

   // is this bitmap transparent
   if (transparent)
   {
   // copy each line of bitmap into destination with transparency
   for (index=0; index < SCREEN_HEIGHT; index++)
       {
       // copy the memory
       for (pixel_x=0; pixel_x < SCREEN_WIDTH; pixel_x++)
           {
           if ((pixel = source_addr[pixel_x])!=0)
               dest_addr[pixel_x] = pixel;

           } // end if

       // advance all the pointers
       dest_addr   += lpitch;
       source_addr += SCREEN_WIDTH;

       } // end for index
   } // end if
   else
   {
   // non-transparent version
   // copy each line of bitmap into destination

   for (index=0; index < SCREEN_HEIGHT; index++)
       {
       // copy the memory
       memcpy(dest_addr, source_addr, SCREEN_WIDTH);

       // advance all the pointers
       dest_addr   += lpitch;
       source_addr += SCREEN_WIDTH;

       } // end for index

   } // end else

   // return success
   return(1);

} // end Copy_Screen

/////////////////////////////////////////////////////////////////////

void Do_Intro(void)
{
// the worlds simples intro

// clear out buffers
DDraw_Fill_Surface(lpddsback, 0);
DDraw_Fill_Surface(lpddsprimary, 0);

// draw in logo screen
Load_Bitmap_File(&bitmap8bit, "ART/starescape8.bmp");

// set the palette to background image palette
Set_Palette(bitmap8bit.palette);

// copy the bitmap to primary buffer
DDraw_Lock_Primary_Surface();
Copy_Screen(bitmap8bit.buffer,primary_buffer, primary_lpitch, 0);
DDraw_Unlock_Primary_Surface();

// unload bitmap file
Unload_Bitmap_File(&bitmap8bit);

Sleep(5000);

// transition to black
Screen_Transitions(SCREEN_DARKNESS,NULL,0);

// clear out buffers
DDraw_Fill_Surface(lpddsback, 0);
DDraw_Fill_Surface(lpddsprimary, 0);

} // end Do_Intro

///////////////////////////////////////////////////////////

int Load_Sound_Music(void)
{
// this function loads all the sounds and music

// load in intro music
if ((intro_music_id = DSound_Load_WAV("SOUND/INTRO.WAV"))==-1)
   return(0);
    
// load the main music
if ((main_music_id = DSound_Load_WAV("SOUND/STARSNG.WAV"))==-1)
	return(0);
// load the main music
//if ((main_music_id = DMusic_Load_MIDI("SOUND/Sinox.mid"))==-1)
//   return(0);

// load get ready
if ((ready_id = DSound_Load_WAV("SOUND/ENTERING1.WAV"))==-1)
   return(0);

// load engines
if ((engines_id = DSound_Load_WAV("SOUND/ENGINES.WAV"))==-1)
   return(0);

// load scream
if ((scream_id = DSound_Load_WAV("SOUND/BREAKUP.WAV"))==-1)
   return(0);

// load jump
if ((jump_id = DSound_Load_WAV("SOUND/FPOWER1.WAV"))==-1)
   return(0);

// load game over
if ((game_over_id = DSound_Load_WAV("SOUND/GAMEOVER.WAV"))==-1)
   return(0);

if ((deactivate_id = DSound_Load_WAV("SOUND/DEACTIVATE1.WAV"))==-1)
   return(0);

if ((station_throb_id = DSound_Load_WAV("SOUND/STATIONTHROB2.WAV"))==-1)
   return(0);


if ((beep0_id = DSound_Load_WAV("SOUND/BEEP3.WAV"))==-1)
   return(0);

if ((beep1_id = DSound_Load_WAV("SOUND/BEEP1.WAV"))==-1)
   return(0);

if ((empty_id = DSound_Load_WAV("SOUND/EMPTY.WAV"))==-1)
   return(0);

// load powerups
if ((ammopu_id = DSound_Load_WAV("SOUND/AMMOPU.WAV"))==-1)
   return(0);

if ((shldpu_id = DSound_Load_WAV("SOUND/SHLDPU.WAV"))==-1)
   return(0);

if ((mappu_id = DSound_Load_WAV("SOUND/DATABASE1.WAV"))==-1)
   return(0);

// load the explosions sounds
if ((station_blow_id = DSound_Load_WAV("SOUND/STATIONBLOW.WAV"))==-1)
   return(0);


// these are the two different source masters
if ((expl_ids[0] = DSound_Load_WAV("SOUND/EXPL1.WAV"))==-1)
    return(0);

if ((expl_ids[1] = DSound_Load_WAV("SOUND/EXPL2.WAV"))==-1)
    return(0);

// now make copies
for (int index=2; index < MAX_EXPL_SOUNDS; index++)
    {
    // replicate sound
    expl_ids[index] = DSound_Replicate_Sound(expl_ids[rand()%2]);  

    } // end for index

// load the plasma weapons sounds
if ((fire_ids[0] = DSound_Load_WAV("SOUND/PULSE.WAV"))==-1)
    return(0);

// now make copies
for (index=1; index < MAX_FIRE_SOUNDS; index++)
    {
    // replicate sound
    fire_ids[index] = DSound_Replicate_Sound(fire_ids[0]);  
    } // end for index

// return sucess
return(1);

} // end Load_Sound_Music

///////////////////////////////////////////////////////////////////

void Load_HUD(void)
{
// this function loads the animation for the hud

int index; // looping variable
	
// load the crescent ship
Load_Bitmap_File(&bitmap8bit, "ART/HUDART8.BMP");

// now create the crescent
Create_BOB(&hud,0,0,24,26,4,
           BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME,
           DDSCAPS_SYSTEMMEMORY);

// load hud frames
for (index=0; index < 4; index++)
     Load_Frame_BOB(&hud,&bitmap8bit,index,index,0,BITMAP_EXTRACT_MODE_CELL);  

// unload data infile
Unload_Bitmap_File(&bitmap8bit);

} // end Load_HUD


///////////////////////////////////////////////////////////////////////////////

int Load_Player(void)
{
// this function loads the animation for the player

int index; // looping variable
	
// load the crescent ship
Load_Bitmap_File(&bitmap8bit, "ART/crescent.bmp");

// set the palette to background image palette
Set_Palette(bitmap8bit.palette);

// save this palette
Save_Palette(game_palette);

// now create the crescent
Create_BOB(&crescent,0,0,64,64,35,
           BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME,
           DDSCAPS_SYSTEMMEMORY);

// load crescent frames
for (index=0; index < 35; index++)
     Load_Frame_BOB(&crescent,&bitmap8bit,index,index%8,index/8,BITMAP_EXTRACT_MODE_CELL);  

// set position
Set_Pos_BOB(&crescent,(SCREEN_WIDTH/2)-crescent.width/2,(SCREEN_HEIGHT/2)-crescent.height/2);

// set starting direction
crescent.varsI[CRESCENT_INDEX_DIR] = 0;

// unload data infile
Unload_Bitmap_File(&bitmap8bit);

// return success
return(1);

} // end Load_Player

////////////////////////////////////////////////////////////////////

void Create_Tables(void)
{
// this function creates all the lookup tables for the game

int ang = 0; // looping var to track angle

// create the 16 sector sin, cos lookup
for (ang=0; ang < 16; ang++)
	{
	float fang = PI*(ang*22.5)/180;

	cos_look16[ang] = -cos(fang+PI/2);
	sin_look16[ang] = -sin(fang+PI/2);

	} // end for ang

} // end Create_Tables

/////////////////////////////////////////////////////////////////////////////////////

float Fast_Distance_2D(float x, float y)
{
// this function computes the distance from 0,0 to x,y with 3.5% error

// first compute the absolute value of x,y
x = fabs(x);
y = fabs(y);

// compute the minimum of x,y
float mn = MIN(x,y);

// return the distance
return(x+y-(mn/2)-(mn/4)+(mn/8));

} // end Fast_Distance_2D

///////////////////////////////////////////////////////////

// WINX GAME PROGRAMMING CONSOLE FUNCTIONS ////////////////

void Game_Reset(void)
{
// this function resets all the game related systems and vars for another run

int index; // looping var

// player state variables
/*player_state       = PLAYER_STATE_ALIVE;
player_score       = 0;   // the score
player_ships       = 3;   // ships left
player_regen_counter = 0; // used to track when to regen
player_damage      = 0;   // damage of player
player_counter     = 0;   // used for state transition tracking
player_regen_count = 0;   // used to regenerate player
player_shield_count = 0;  // used to display shields  */

// menu_up = 0;             // flags if the main menu is up
// intro_done = 0;          // flags if intro has played already

// positional and state info for player
player_x=0;
player_y=0;
player_dx=0;
player_dy=0; 
player_xv=0; 
player_yv=0;
vel = 0;
crescent.varsI[CRESCENT_INDEX_DIR] = 0;
game_state = GAME_STATE_RESTART;   // initial game state
station_id  = -1;

//ready_counter = 0,        // used to draw a little "get ready"
//ready_state   = 0;

// call all the reset funcs
Init_Reset_Particles();
Reset_Aradas();
Reset_Kraits();
Reset_Laziras();
Reset_Zidegars();
Reset_Miranu();
Reset_Rocks();
Reset_Plasma();
Reset_Bursts();
Reset_Stations();
Reset_Powerups();

// start the rocks
Seed_Rocks();

//Start the ships
for (index=0; index < NUM_ACTIVE_ARADAS; index++)
    Start_Arada();

for (index=0; index < NUM_ACTIVE_LAZIRAS; index++)
    Start_Lazira();

for (index=0; index < NUM_ACTIVE_ZIDEGARS; index++)
    Start_Zidegar();

for (index=0; index < NUM_ACTIVE_KRAITS; index++)
    Start_Krait();

for (index=0; index < NUM_ACTIVE_MIRANU; index++)
	Start_Miranu();

for (index=0; index < NUM_ACTIVE_STATIONS; index++)
    Start_Station();
 
} // end Game_Reset

////////////////////////////////////////////////////////////

int Game_Init(void *parms)
{


// this function is where you do all the initialization 
// for your game

//int index;         // looping var
//char filename[80]; // used to build up files names

//mono.clear();
//mono.print("\ndebugger on-line\n");

// seed random number generate
srand(Start_Clock());

// create all lookup tables
Create_Tables();

// initialize directdraw
DDraw_Init(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP);

// initialize directsound
DSound_Init();

// initialize Directinput
DInput_Init();

// initialize all input devices
DInput_Init_Keyboard();
 
// load all the sound and music
Load_Sound_Music();

// initialize the plasma pulses
Init_Plasma();

// initialize all the stars
Init_Stars();

// init rocks
Init_Rocks();
Seed_Rocks();

// init all the explosions
Init_Bursts();

// initialize all the ships
Init_Aradas();
Init_Kraits();
Init_Laziras();
Init_Zidegars();
Init_Miranu();

// initialize all the stations
Init_Stations();

// initialize the powerups
Init_Powerups();

// intialize particle system
Init_Reset_Particles();

// set clipping rectangle to screen extents so objects dont
// mess up at edges
RECT screen_rect = {0,0,screen_width,screen_height};
lpddclipper = DDraw_Attach_Clipper(lpddsback,1,&screen_rect);

// hide the mouse
ShowCursor(FALSE);

// start the intro music
DSound_Play(intro_music_id);

// do the introduction
Do_Intro();

// load the player
Load_Player();

// load the hud art
Load_HUD();

Init_Message();
Send_Message("What is this strange system I'm in?", PLAYER_FACE);

// return success
return(1);

} // end Game_Init

///////////////////////////////////////////////////////////

int Game_Shutdown(void *parms)
{
// this function is where you shutdown your game and
// release all resources that you allocated

// delete all the explosions
Delete_Bursts();
    
// delete the player
Destroy_BOB(&crescent);

// delete all the rocks
Delete_Rocks();

// delete all the plasma pulses
Delete_Plasma();

// delete all the stations
Delete_Stations();

// delete all the ships
Delete_Aradas();
Delete_Kraits();
Delete_Laziras();
Delete_Zidegars();
Delete_Miranu();

// shutdown directdraw
DDraw_Shutdown();

// unload sounds
DSound_Delete_All_Sounds();

// shutdown directsound
DSound_Shutdown();

// release all input devices
DInput_Release_Keyboard();

// return success
return(1);
} // end Game_Shutdown

///////////////////////////////////////////////////////////

int Game_Main(void *parms)
{
// this is the workhorse of your game it will be called
// continuously in real-time this is like main() in C
// all the calls for you game go here!

//int  index;                  // looping var
//int  dx,dy;                  // general deltas used in collision detection
float xv=0, yv=0;            // used to compute velocity 
static int engines_on = 0;   // tracks state of engines each frame 
static int andre_up = 0;


// what state is the main loop in?
if (game_state == GAME_STATE_INIT)
{
// reset everything

// transition into menu state
 game_state = GAME_STATE_MENU;
Draw_Message();
// transition into run state
game_state = GAME_STATE_RESTART;
} // end game_state is game init
else
if (game_state == GAME_STATE_MENU)
{

game_state = GAME_STATE_RESTART;

} // end if in menu state
else 
if (game_state == GAME_STATE_RESTART)
{
// start the main music
DSound_Play(main_music_id, DSBPLAY_LOOPING);

// restart the game
Game_Reset();

// set to run state
game_state = GAME_STATE_RUNNING;


} // end if restart
else
if (game_state == GAME_STATE_RUNNING)
{

// start the timing clock
Start_Clock();

// clear the drawing surface
DDraw_Fill_Surface(lpddsback, 0);

// read all input devices
DInput_Read_Keyboard();

// only process player if alive
if (player_state == PLAYER_STATE_ALIVE)
{

if  (keyboard_state[DIK_J])//cheat for testing
    {
		if(GotMap == 0)
		{
			Send_Message("I can't jump yet, I don't have a map.", PLAYER_FACE);
			DSound_Play(empty_id);
		}
		else if (GotMap == 1)
		{
			Send_Message("Here we go!!", PLAYER_FACE);
			DSound_Play(jump_id);
			player_state = PLAYER_STATE_GAME_WON;
		}
    } // end if

	
// test if player is moving
if (keyboard_state[DIK_RIGHT])
    {
    // rotate player to right
    if (++crescent.varsI[CRESCENT_INDEX_DIR] > 15)
		crescent.varsI[CRESCENT_INDEX_DIR] = 0;
	
    } // end if
//else
if (keyboard_state[DIK_LEFT])
    {
    // rotate player to left
    if (--crescent.varsI[CRESCENT_INDEX_DIR] <  0)
	 	crescent.varsI[CRESCENT_INDEX_DIR] = 15;

    } // end if

// vertical/speed motion

/*if (keyboard_state[DIK_DOWN])
    {
    // move player back
    xv = -cos_look16[crescent.varsI[CRESCENT_INDEX_DIR]];
	yv = -sin_look16[crescent.varsI[CRESCENT_INDEX_DIR]];

    } // end if*/

if (keyboard_state[DIK_UP])
    {
    // move player forward
    xv = cos_look16[crescent.varsI[CRESCENT_INDEX_DIR]];
	yv = sin_look16[crescent.varsI[CRESCENT_INDEX_DIR]];

    // test to turn on engines
    if (!engines_on)
       DSound_Play(engines_id,DSBPLAY_LOOPING);

	// set engines to on
	engines_on = 1;

    Start_Particle(PARTICLE_TYPE_FADE, PARTICLE_COLOR_BLUE, 3, 
                   player_x+RAND_RANGE(-2,2), player_y+RAND_RANGE(-2,2), (-int(player_xv)>>3), (-int(player_yv)>>3));
    
    } // end if
else
if (engines_on)
   {
   // reset the engine on flag and turn off sound
   engines_on = 0;

   // turn off the sound
 
   DSound_Stop_Sound(engines_id);
   } // end if



// toggle huds
if (keyboard_state[DIK_H] && !huds_debounce)
   {
   huds_debounce = 1;
   huds_on = (huds_on) ? 0 : 1;

   DSound_Play(beep1_id);

   } // end if

if (!keyboard_state[DIK_H])
   huds_debounce = 0;

// toggle scanner
if (keyboard_state[DIK_S] && !scanner_debounce)
   {
   scanner_debounce = 1;
   scanner_on = (scanner_on) ? 0 : 1;

   DSound_Play(beep1_id);

   } // end if

if (!keyboard_state[DIK_S])
   scanner_debounce = 0;



// add velocity change to player's velocity
player_xv+=xv;
player_yv+=yv;

// test for maximum velocity
vel = Fast_Distance_2D(player_xv, player_yv);

if (vel >= MAX_PLAYER_SPEED)
   {
   // re-compute velocity vector by normalizing then re-scaling
   player_xv = (MAX_PLAYER_SPEED-1)*player_xv/vel;
   player_yv = (MAX_PLAYER_SPEED-1)*player_yv/vel;
   } // end if

// add in frictional coefficient

// move player, note that these are in world coords
player_x+=player_xv;
player_y+=player_yv;

// wrap player to universe
if (player_x > UNIVERSE_MAX_X)
   {
   // reset pos
   player_x = UNIVERSE_MIN_X;

   // set warp flag
   } // end if
else
if (player_x < UNIVERSE_MIN_X)
   {
   // reset pos
   player_x = UNIVERSE_MAX_X;

   // set warp flag
   } // end if

// y coords
if (player_y > UNIVERSE_MAX_Y)
   {
   // reset pos
   player_y = UNIVERSE_MIN_Y;

   // set warp flag
   } // end if
else
if (player_y < UNIVERSE_MIN_Y)
   {
   // reset pos
   player_y = UNIVERSE_MAX_Y;

   // set warp flag
   } // end if


// test if player is firing

if ((keyboard_state[DIK_LCONTROL] || keyboard_state[DIK_SPACE]))
	{
   // if (player_ammo > 0)
      // {
    	// compute plasma velocity vector
		
    float plasma_xv = cos_look16[crescent.varsI[CRESCENT_INDEX_DIR]]*PLASMA_SPEED;
	float plasma_yv = sin_look16[crescent.varsI[CRESCENT_INDEX_DIR]]*PLASMA_SPEED;
	if (WeaponUpgradeLevel%2)
	Fire_Plasma(player_x,player_y,
	            player_xv+plasma_xv,
			    player_yv+plasma_yv,
				PLASMA_ANIM_PLAYER);
for(int index = 1; index<WeaponUpgradeLevel; index+=2){
	// fire the plasma taking into consideration ships's velocity
	Fire_Plasma(player_x,player_y,
	            player_xv+cos_look16[(crescent.varsI[CRESCENT_INDEX_DIR]-index)%16]*PLASMA_SPEED,
			    player_yv+sin_look16[(crescent.varsI[CRESCENT_INDEX_DIR]-index)%16]*PLASMA_SPEED,
				PLASMA_ANIM_PLAYER);
	Fire_Plasma(player_x,player_y,
	            player_xv+cos_look16[(crescent.varsI[CRESCENT_INDEX_DIR]+index)%16]*PLASMA_SPEED,
			    player_yv+sin_look16[(crescent.varsI[CRESCENT_INDEX_DIR]+index)%16]*PLASMA_SPEED,
				PLASMA_ANIM_PLAYER);
}
	
/*
 	    // fire the plasma taking into consideration ships's velocity
        Fire_Plasma(player_x,player_y,
		            player_xv+plasma_xv,
				    player_yv+plasma_yv, PLASMA_ANIM_PLAYER);
/**/

        // use up some ammo
        /*if (player_ammo > 0)
        player_ammo--;*/

	   // } // end if
   /* else
        {
        DSound_Play(empty_id);
        } // end else*/

    } // end if

// do bounds check


// regenerate player
if (++player_regen_counter >= PLAYER_REGEN_COUNT)
    {
    // regenerate the player a bit
    if (player_damage >0 )
        player_damage--;

    // reset counter
    player_regen_counter = 0;

    } // end if


// test for dying state transition
if (player_damage >= 100 && player_state == PLAYER_STATE_ALIVE)
    {
    // set damage to 100
    player_damage = 100;

    // kill player
    player_state = PLAYER_STATE_DYING;
    player_ships--;

    // set counter to 0
    player_counter = 0;

    // turn engines off
	engines_on = 0;

    // start scream
//    DSound_Play(scream_id);
    Send_Message("I'm going down!",PLAYER_FACE);
    } // end if


} // end if player alive
else
if (player_state == PLAYER_STATE_DYING)
   { 
   // player is dying

   // start random bursts
   int bwidth  = 16 + rand()%64;
   
   if ((rand()%4)==1)
        Start_Burst(player_x+rand()%40, player_y+rand()%8,
                   bwidth, (bwidth >> 2) + (bwidth >> 1),
                   -4+rand()%8,2+rand()%4);    

   // get jiggie with it
   crescent.varsI[CRESCENT_INDEX_DIR] += (RAND_RANGE(-1,1));

   if (crescent.varsI[CRESCENT_INDEX_DIR] > 15)
       crescent.varsI[CRESCENT_INDEX_DIR] = 0;
   else
   if (crescent.varsI[CRESCENT_INDEX_DIR] < 0)
      crescent.varsI[CRESCENT_INDEX_DIR] = 15;

   // update state counter
   if (++player_counter > 150)
      {
      // set state to invincible  
      player_state = PLAYER_STATE_INVINCIBLE;

      // reset counter
      player_counter = 0;

      // reset damage
      player_damage = 0;
      } // end if


   } // end if
else
if (player_state == PLAYER_STATE_INVINCIBLE)
   {
   // player is invincible
 
   // flicker shields and play energizing sound

   // update state counter
   if (++player_counter > 70)
      {
      // set state to invincible  
      player_state = PLAYER_STATE_ALIVE;

      // reset counter
      player_counter = 0;

      // reset player position
      player_x = 0;
      player_y = 0;
      player_dx=0;
      player_dy=0; 
      player_xv=0; 
      player_yv=0;
      crescent.varsI[CRESCENT_INDEX_DIR] = 0;

      // decrease player ships
      if (player_ships == 0)
         {
         // change state to dead, reset all vars
         player_state   = PLAYER_STATE_DEAD;
         player_counter = 0;
         ready_state    = 1;
         ready_counter  = 0;

         // turn engines
         DSound_Stop_Sound(engines_id);

         // play game over sound
         DSound_Play(game_over_id);

         } // end if       

      } // end if

    // start ready again
    if ((player_counter == 20) && (player_ships > 0))
      {
      // reset to ready
      ready_counter = 0, 
      ready_state   = 0;
      } // end if
   
   } // end if

// end player sub states

// move the powerups 
Move_Powerups();

// move the stations
Move_Stations();

// move the ships
Move_Aradas();
Move_Kraits();
Move_Laziras();
Move_Zidegars();
Move_Miranu();

// move the asteroids
Move_Rocks();

// move the stars
Move_Stars();

// move the particles
Move_Particles();

// move the explosions
Move_Bursts();

// draw the stars
Draw_Stars();

// draw the powerups
Draw_Powerups();

// draw the stations
Draw_Stations();

// draw the ships
Draw_Aradas();
Draw_Kraits();
Draw_Laziras();
Draw_Zidegars();
Draw_Miranu();

// draw the rocks
Draw_Rocks();

// draw the particles
Draw_Particles();

// draw the plasma
Draw_Plasma();

// move the plasma
Move_Plasma();

// draw the player if alive
if (player_state == PLAYER_STATE_ALIVE || player_state == PLAYER_STATE_DYING)
	{	
	// set the current frame
    if (engines_on)
		crescent.curr_frame = crescent.varsI[CRESCENT_INDEX_DIR] + 16*(rand()%2);
	else
		crescent.curr_frame = crescent.varsI[CRESCENT_INDEX_DIR];

	// draw the bob
    Draw_BOB(&crescent,lpddsback);

    // draw the shields
    if (player_shield_count > 0)
       {
       player_shield_count--;

       // select shield frame
       crescent.curr_frame = 34-player_shield_count;    
	   
       // draw the bob
       Draw_BOB(&crescent,lpddsback);

       } // end if

	} // end if

// draw explosions last
Draw_Bursts();

// draw the score and ships left
if (huds_on)
    Draw_Info();

if (scanner_on)
    Draw_Scanner();
	
if (message_timer > 0)
    Draw_Message();

// last state here so it will overlay
if (player_state == PLAYER_STATE_DEAD)
   {
   // player is dead
   ready_state   = 1;
   ready_counter = 0;

   // player is done!
   Draw_Text_GDI("G A M E   O V E R",(SCREEN_WIDTH/2) - 8*(strlen("G A M E   O V E R")>>1),SCREEN_HEIGHT/2,RGB(0,255,0),lpddsback);

   } // end if

if (player_state == PLAYER_STATE_GAME_WON)
   {
   // player is dead
	ready_state   = 1;
	ready_counter = 0;
	
	Draw_Text_GDI("Y O U   W I N ! !",(SCREEN_WIDTH/2) - 8*(strlen("Y O U   W I N ! !")>>1),SCREEN_HEIGHT/2,RGB(0,255,0),lpddsback);

   } // end if

// draw get ready?
if (ready_state==0)
   {
   // test if counter is 10 for voice
//   if (ready_counter==10)
//       DSound_Play(ready_id);

   // draw text
   Draw_Text_GDI("E N T E R I N G   V I N J A G G A   S Y S T E M ",320-8*strlen("E N T E R I N G   S Y S T E M!")/2, 200,RGB(0,rand()%255,0),lpddsback);

   // increment counter
   if (++ready_counter > 60)
      {
      // set state to ready
      ready_state = 1;
      ready_counter = 0;

      } // end if

   } // end if

// test for me!
/*if (keyboard_state[DIK_LALT] && keyboard_state[DIK_RALT] && keyboard_state[DIK_A])
   {
   // lock primary
   DDraw_Lock_Back_Surface();   
   Draw_Bitmap(&andre,back_buffer, back_lpitch,0);        
   // release the primary buffer
   DDraw_Unlock_Back_Surface();   
   andre_up = 1;
   } // end if
else 
   {
   andre_up = 0;
   Set_Palette(game_palette);
   } // end else

if (andre_up == 1)
   {
   Set_Palette(andre_palette);
   andre_up = 2;
   } // end if
/**/

// flip the surfaces
DDraw_Flip();

// sync to 30ish fps
Wait_Clock(30);

// check of user is trying to exit
if (++player_counter > 10)
if (keyboard_state[DIK_ESCAPE])
   {
   // send game back to menu state
   game_state = GAME_STATE_EXIT;

   // stop all sounds
   DSound_Stop_Sound(engines_id);

   } // end if


} // end if game running
else
if (game_state == GAME_STATE_PAUSED)
   {
    // pause game
    if (keyboard_state[DIK_P] && !pause_debounce)
       {
       pause_debounce = 1;
       game_paused = (game_paused) ? 0 : 1;

       DSound_Play(beep1_id);

       } // end if

    if (!keyboard_state[DIK_P])
       game_paused = 0;

    if (game_paused)
       {
       // draw text
       Draw_Text_GDI("G A M E  P A U S E D  -- P R E S S  <P>",320-8*strlen("G A M E  P A U S E D  -- P R E S S  <P>")/2, 200, RGB(255,0,0),lpddsback);
       } // end if
    else
       game_state = GAME_STATE_RUNNING;


   } // end if
else
if (game_state == GAME_STATE_EXIT)
   {
   // this is the exit state, called just once
   PostMessage(main_window_handle, WM_DESTROY,0,0);
   game_state = GAME_STATE_WAITING_FOR_EXIT;

   // fade to black
   Screen_Transitions(SCREEN_DARKNESS, NULL, 0);

   // clear out buffers
   DDraw_Fill_Surface(lpddsback, 0);
   DDraw_Fill_Surface(lpddsprimary, 0);

   } // end if
else
if (game_state == GAME_STATE_WAITING_FOR_EXIT)
   {
   // wait here in safe state

   } // end wait

// return success
return(1);

} // end Game_Main

//end