
void Init_Zidegars(void)
{
// this function loads and initializes the zidegars to a known state
	
int frame;  // looping var

// create the first bob
Create_BOB(&zidegars[0],0,0,64,64,32,
            BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME,
            DDSCAPS_SYSTEMMEMORY);

	// load animation frames 
	Load_Bitmap_File(&bitmap8bit, "ART/zidegar.bmp");

	for (frame=0; frame < 31; frame++)
		Load_Frame_BOB(&zidegars[0],&bitmap8bit,frame,frame%8,frame/8,BITMAP_EXTRACT_MODE_CELL);  

    // unload data infile
    Unload_Bitmap_File(&bitmap8bit);


// set state to off
zidegars[0].state                       = ZIDEGAR_STATE_DEAD;
zidegars[0].varsI[INDEX_ZIDEGAR_DIR]    = 0;
zidegars[0].varsI[INDEX_ZIDEGAR_DAMAGE] = 0;

// make copies
for (int ship=1; ship < MAX_ZIDEGARS; ship++)
    {
    memcpy(&zidegars[ship], &zidegars[0], sizeof(BOB));
    } // end for zidegar

} // end Init_Aradas

//////////////////////////////////////////////////////////////////////////////////////

void Reset_Zidegars(void)
{
// this function resets all the zidegars

for (int ship=0; ship < MAX_ZIDEGARS; ship++)
    {
    zidegars[ship].state                       = ZIDEGAR_STATE_DEAD;
    zidegars[ship].varsI[INDEX_ZIDEGAR_DIR]    = 0;
    zidegars[ship].varsI[INDEX_ZIDEGAR_DAMAGE] = 0;
    } // end for zidegar

} // end Reset_Zidegars

///////////////////////////////////////////////////////////

void Start_Zidegar(int override=0, int x=0, int y=0, int dir=0)
{
// this functions starts an zidegar, note that if override = 1
// then the function uses the sent data otherwise it's random

// first find an available zidegar
for (int index=0; index < MAX_ZIDEGARS; index++)
	{
	// is this one dead
	if (zidegars[index].state  == ZIDEGAR_STATE_DEAD)
		{
		// position the zidegar on a circle ring randomly from player

		int ang = rand()%16;
		float xpos = ZIDEGAR_RANGE_RING*cos_look16[ang];
		float ypos = ZIDEGAR_RANGE_RING*sin_look16[ang];

        // set position
		zidegars[index].varsI[INDEX_WORLD_X] = player_x+xpos;
		zidegars[index].varsI[INDEX_WORLD_Y] = player_y+ypos;

		// set direction to right
		zidegars[index].varsI[INDEX_ZIDEGAR_DIR] = ang;

		// set velocity
		zidegars[index].xv = 4+rand()%8;
		zidegars[index].yv = RAND_RANGE(-4,4);

		
		// set remaining state variables
		zidegars[index].state  = ZIDEGAR_STATE_ALIVE;
        zidegars[index].varsI[INDEX_ZIDEGAR_DIR] = 0;
        zidegars[index].varsI[INDEX_ZIDEGAR_DAMAGE] = 0;

		// done so exit
		return;

		} // end if
	
	} // end for index

} // end Start_Zidegar

////////////////////////////////////////////////////////////

void Move_Zidegars(void)
{
// this function moves all the zidegars and does the ai

for (int index=0; index < MAX_ZIDEGARS; index++)
    {
    // test if its alive
    if (zidegars[index].state == ZIDEGAR_STATE_ALIVE)
        {
        // move the zidegar
        zidegars[index].varsI[INDEX_WORLD_X]+=zidegars[index].xv;
        zidegars[index].varsI[INDEX_WORLD_Y]+=zidegars[index].yv;
          
        // test for boundaries
        if (zidegars[index].varsI[INDEX_WORLD_X] > UNIVERSE_MAX_X)
           {
  		   zidegars[index].varsI[INDEX_WORLD_X] = UNIVERSE_MIN_X;
           } // end if
		else
		if (zidegars[index].varsI[INDEX_WORLD_X] < UNIVERSE_MIN_X )
           {
  		   zidegars[index].varsI[INDEX_WORLD_X] = UNIVERSE_MAX_X;
           } // end if	    

		if (zidegars[index].varsI[INDEX_WORLD_Y] > UNIVERSE_MAX_Y)
           {
		   zidegars[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MIN_Y;
           } // end if
		else
		if (zidegars[index].varsI[INDEX_WORLD_Y] < UNIVERSE_MIN_Y ) 
           {
		   zidegars[index].varsI[INDEX_WORLD_Y] = UNIVERSE_MAX_Y;
           } // end if


       // leave particle trail
    if (zidegars[index].varsI[INDEX_ZIDEGAR_DAMAGE] > (MAX_ZIDEGAR_DAMAGE >> 2) && 
       (rand()%3)==1)
        {
        Start_Particle(PARTICLE_TYPE_FLICKER, PARTICLE_COLOR_WHITE, 30+rand()%25, 
                      zidegars[index].varsI[INDEX_WORLD_X]+RAND_RANGE(-4,4), 
                      zidegars[index].varsI[INDEX_WORLD_Y]+RAND_RANGE(-4,4), 
                      (zidegars[index].xv*.125), (zidegars[index].yv*.125));
    
        Start_Particle(PARTICLE_TYPE_FADE, PARTICLE_COLOR_RED, 5, 
                      zidegars[index].varsI[INDEX_WORLD_X]+RAND_RANGE(-4,4), 
                      zidegars[index].varsI[INDEX_WORLD_Y]+RAND_RANGE(-4,4), 
                      (zidegars[index].xv*.125), (zidegars[index].yv*.125));


    
        } // end if

		// do ai
		
		// tracking algorithm

        // compute vector toward player
        float vx = (player_x - cos_look16[crescent.varsI[CRESCENT_INDEX_DIR]]*200 ) - zidegars[index].varsI[INDEX_WORLD_X];
        float vy = (player_y - sin_look16[crescent.varsI[CRESCENT_INDEX_DIR]]*200 )- zidegars[index].varsI[INDEX_WORLD_Y];

        // normalize vector (sorta :)
        float length = Fast_Distance_2D(vx,vy);


        // only track if reasonable close
        if ((length < MIN_ZIDEGAR_TRACKING_DIST))
            {
            vx=zidegar_tracking_rate*vx/length;
            vy=zidegar_tracking_rate*vy/length;

            // add velocity vector to current velocity
            zidegars[index].xv+=vx;
            zidegars[index].yv+=vy;

            // add a little noise
            if ((rand()%10)==1)
                {
                vx = RAND_RANGE(-1,1);
                vy = RAND_RANGE(-1,1);
                zidegars[index].xv+=vx;
                zidegars[index].yv+=vy;
                } // end if

            // test velocity vector of zidegars
            length = Fast_Distance_2D(zidegars[index].xv, zidegars[index].yv);

            // test for velocity overflow and slow
            if (length > MAX_ZIDEGAR_VELOCITY)
                {
                // slow down
                zidegars[index].xv*=0.75;
                zidegars[index].yv*=0.75;

                } // end if

           
        else
            {
            // add a random velocity component
            if ((rand()%30)==1)
                {
                vx = RAND_RANGE(-2,2);
                vy = RAND_RANGE(-2,2);

                // add velocity vector to current velocity
                zidegars[index].xv+=vx;
                zidegars[index].yv+=vy;

                // test velocity vector of zidegars
                length = Fast_Distance_2D(zidegars[index].xv, zidegars[index].yv);

                // test for velocity overflow and slow
                if (length > MAX_ZIDEGAR_VELOCITY)
                    {
                    // slow down
                    zidegars[index].xv*=0.75;
                    zidegars[index].yv*=0.75;

                    } // end if
        
                } // end if
           
            } // end else



            // turn turret always
            // compute d0, d1, d2   

            // first create a vector point in the direction of the turret
            
            // compute current turrent vector
            int tdir1 = zidegars[index].varsI[INDEX_ZIDEGAR_DIR];
            
            float d1x = zidegars[index].varsI[INDEX_WORLD_X] + cos_look16[tdir1]*32;
            float d1y = zidegars[index].varsI[INDEX_WORLD_Y] + sin_look16[tdir1]*32;

            // compute turret vector plus one
            int tdir2 = zidegars[index].varsI[INDEX_ZIDEGAR_DIR]+1;
            
            if (tdir2 > 15) 
                tdir2 = 0;

            float d2x = zidegars[index].varsI[INDEX_WORLD_X] + cos_look16[tdir2]*32;
            float d2y = zidegars[index].varsI[INDEX_WORLD_Y] + sin_look16[tdir2]*32;
            
            // compute turret vector minus one
            int tdir0 = zidegars[index].varsI[INDEX_ZIDEGAR_DIR]-1;
            
            if (tdir0 < 0) 
                tdir0=15;

            float d0x = zidegars[index].varsI[INDEX_WORLD_X] + cos_look16[tdir0]*32;
            float d0y = zidegars[index].varsI[INDEX_WORLD_Y] + sin_look16[tdir0]*32;

            // now find the min dist
            float dist0 = Fast_Distance_2D(player_x - d0x, player_y - d0y);
            float dist1 = Fast_Distance_2D(player_x - d1x, player_y - d1y);
            float dist2 = Fast_Distance_2D(player_x - d2x, player_y - d2y);

            if (dist0 < dist2 && dist0 < dist1)
                {
                // the negative direction is best        
                zidegars[index].varsI[INDEX_ZIDEGAR_DIR] = tdir0;

                } // end if
            else
            if (dist2 < dist0 && dist2 < dist1)
                {
                // the positive direction is best
                zidegars[index].varsI[INDEX_ZIDEGAR_DIR] = tdir2;

                } // end if

            // test for firing
            if ( (rand()%10)==1 &&
				(Fast_Distance_2D(player_x-zidegars[index].varsI[INDEX_WORLD_X],
			                 player_y-zidegars[index].varsI[INDEX_WORLD_Y]) < ZIDEGAR_MIN_ATTACK_RANGE) )
                {   
					int plasma_x = zidegars[index].varsI[INDEX_WORLD_X];
                    int plasma_y = zidegars[index].varsI[INDEX_WORLD_Y];


                    float plasma_xv = zidegars[index].xv+cos_look16[zidegars[index].varsI[INDEX_ZIDEGAR_DIR]]*PLASMA_SPEED_ZIDEGAR;
	                float plasma_yv = zidegars[index].yv+sin_look16[zidegars[index].varsI[INDEX_ZIDEGAR_DIR]]*PLASMA_SPEED_ZIDEGAR;

                    // fire weapon
                    Fire_Plasma(plasma_x, plasma_y, plasma_xv,plasma_yv,PLASMA_ANIM_ENEMY);

				} // end if fire

	
		}	//if close
    } // end if alive
	} // end for index
} // end Move_Zidegars

///////////////////////////////////////////////////////////

void Draw_Zidegars(void)
{
// this function moves all the zidegars

for (int index=0; index < MAX_ZIDEGARS; index++)
    {
    // test if plasma pulse is in flight
    if (zidegars[index].state == ZIDEGAR_STATE_ALIVE)
        {
        // transform to screen coords
        zidegars[index].x = zidegars[index].varsI[INDEX_WORLD_X] - (zidegars[index].width >> 1) - player_x + (SCREEN_WIDTH/2);
        zidegars[index].y = zidegars[index].varsI[INDEX_WORLD_Y] - (zidegars[index].height >> 1) - player_y + (SCREEN_HEIGHT/2);

        // select the proper frame
		zidegars[index].curr_frame = (zidegars[index].varsI[INDEX_ZIDEGAR_DIR] + 16*(rand()%2) );

        // draw the zidegar
        Draw_BOB(&zidegars[index],lpddsback);
         
        } // end if

    } // end for index

} // end Draw_Zidegars

////////////////////////////////////////////////////////////

void Delete_Zidegars(void)
{
// this function simply deletes all memory and surfaces
// related to the zidegars

for (int index=0; index < MAX_ZIDEGARS; index++)
    Destroy_BOB(&zidegars[index]);

} // end Delete_Zidegars

/////////////////////////////////////////////////////
